@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -119,7 +119,7 @@ void CheckBufferOverrun::possibleReadlinkBufferOverrunError(const Token* tok, co
 
 void CheckBufferOverrun::strncatUsageError(const Token *tok)
 {
-    if (_settings && !_settings->isEnabled("style"))
+    if (_settings && !_settings->isEnabled("warning"))
         return;
 
     reportError(tok, Severity::warning, "strncatUsage",
@@ -147,7 +147,7 @@ void CheckBufferOverrun::pointerOutOfBoundsError(const Token *tok, const std::st
 
 void CheckBufferOverrun::sizeArgumentAsCharError(const Token *tok)
 {
-    if (_settings && !_settings->isEnabled("style"))
+    if (_settings && !_settings->isEnabled("warning"))
         return;
     reportError(tok, Severity::warning, "sizeArgumentAsChar", "The size argument is given as a char constant.");
 }
@@ -176,6 +176,11 @@ void CheckBufferOverrun::bufferNotZeroTerminatedError(const Token *tok, const st
     reportError(tok, Severity::warning, "bufferNotZeroTerminated", errmsg, true);
 }
 
+void CheckBufferOverrun::argumentSizeError(const Token *tok, const std::string &functionName, const std::string &varname)
+{
+    reportError(tok, Severity::warning, "argumentSize", "The array '" + varname + "' is too small, the function '" + functionName + "' expects a bigger one.");
+}
+
 //---------------------------------------------------------------------------
 
 
@@ -375,15 +380,13 @@ static bool for3(const Token * const tok,
                  const bool maxMinFlipped)
 {
     assert(tok != 0);
-    if (Token::Match(tok, "%varid% += %num% )", varid) ||
-        Token::Match(tok, "%varid%  = %num% + %varid% )", varid)) {
+    if (Token::Match(tok, "%varid%  = %num% + %varid% )", varid)) {
         if (!for_maxvalue(tok->tokAt(2), min_value, max_value))
             return false;
     } else if (Token::Match(tok, "%varid% = %varid% + %num% )", varid)) {
         if (!for_maxvalue(tok->tokAt(4), min_value, max_value))
             return false;
-    } else if (Token::Match(tok, "%varid% -= %num% )", varid) ||
-               Token::Match(tok, "%varid%  = %num% - %varid% )", varid)) {
+    } else if (Token::Match(tok, "%varid% = %num% - %varid% )", varid)) {
         if (!for_maxvalue(tok->tokAt(2), min_value, max_value))
             return false;
     } else if (Token::Match(tok, "%varid% = %varid% - %num% )", varid)) {
@@ -418,7 +421,7 @@ static bool for_bailout(const Token * const tok1, unsigned int varid)
     for (const Token *loopTok = tok1; loopTok && loopTok != tok1->link(); loopTok = loopTok->next()) {
         if (loopTok->varId() == varid) {
             // Counter variable used inside loop
-            if (Token::Match(loopTok->next(), "+=|-=|++|--|=") ||
+            if (Token::Match(loopTok->next(), "++|--|=") ||
                 (loopTok->previous()->type() == Token::eIncDecOp)) {
                 return true;
             }
@@ -433,10 +436,32 @@ void CheckBufferOverrun::parse_for_body(const Token *tok, const ArrayInfo &array
     const std::string pattern = (arrayInfo.varid() ? std::string("%varid%") : arrayInfo.varname()) + " [ " + strindex + " ]";
 
     for (const Token* tok2 = tok; tok2 && tok2 != tok->link(); tok2 = tok2->next()) {
-        // TODO: try to reduce false negatives. This is just a quick fix
-        // for TestBufferOverrun::array_index_for_question
-        if (tok2->str() == "?")
-            break;
+        // TestBufferOverrun::array_index_for_question
+        if (tok2->str() == "?") {
+            // does condition check counter variable?
+            bool usesCounter = false;
+            const Token *tok3 = tok2->previous();
+            while (Token::Match(tok3, "%comp%|%num%|%var%|)")) {
+                if (tok3->str() == strindex) {
+                    usesCounter = true;
+                    break;
+                }
+                tok3 = tok3->previous();
+            }
+
+            // If strindex is used in the condition then skip the
+            // conditional expressions
+            if (usesCounter) {
+                while (tok2 && !Token::Match(tok2, "[)],;]")) {
+                    if (tok2->str() == "(" || tok2->str() == "[")
+                        tok2 = tok2->link();
+                    tok2 = tok2->next();
+                }
+                if (!tok2)
+                    break;
+                continue;
+            }
+        }
 
         if (Token::simpleMatch(tok2, "for (") && Token::simpleMatch(tok2->next()->link(), ") {")) {
             const Token *endpar = tok2->next()->link();
@@ -566,7 +591,7 @@ void CheckBufferOverrun::checkFunctionParameter(const Token &tok, unsigned int p
 
         // Parse function call. When a ',' is seen, arg is decremented.
         // if arg becomes 1 then the current function parameter is the wanted parameter.
-        // if arg becomes 1000 then multiply current and next argument.
+        // if arg becomes 1001 then multiply current and next argument.
         const Token *tok2 = tok.tokAt(2)->nextArgument();
         if (arg == 3)
             tok2 = tok2->nextArgument();
@@ -600,7 +625,8 @@ void CheckBufferOverrun::checkFunctionParameter(const Token &tok, unsigned int p
     // Calling a user function?
     // only 1-dimensional arrays can be checked currently
     else if (arrayInfo.num().size() == 1) {
-        const Function* func = _tokenizer->getSymbolDatabase()->findFunctionByName(tok.str(), tok.scope());;
+        const Function* func = tok.function();
+
         if (func && func->hasBody) {
             // Get corresponding parameter..
             const Variable* parameter = func->getArgumentVar(par-1);
@@ -643,7 +669,7 @@ void CheckBufferOverrun::checkFunctionParameter(const Token &tok, unsigned int p
                         Token::Match(ftok, "%var% --"))
                         break;
 
-                    if (Token::Match(ftok->previous(), "=|;|{|}|%op% %var% [ %num% ]")) {
+                    if (Token::Match(ftok->previous(), ";|{|}|%op% %var% [ %num% ]")) {
                         const MathLib::bigint index = MathLib::toLongNumber(ftok->strAt(2));
                         if (index >= 0 && arrayInfo.num(0) > 0 && index >= arrayInfo.num(0)) {
                             std::list<const Token *> callstack2(callstack);
@@ -666,6 +692,38 @@ void CheckBufferOverrun::checkFunctionParameter(const Token &tok, unsigned int p
             }
         }
     }
+
+    // Check 'float x[10]' arguments in declaration
+    if (_settings->isEnabled("warning")) {
+        const Function* func = tok.function();
+
+        // If argument is '%type% a[num]' then check bounds against num
+        if (func) {
+            const Variable* argument = func->getArgumentVar(par-1);
+            const Token *nameToken;
+            if (argument && Token::Match(argument->typeStartToken(), "%type% %var% [ %num% ] [,)[]")
+                && (nameToken = argument->nameToken()) != NULL) {
+                const Token *tok2 = nameToken->next();
+
+                MathLib::bigint argsize = _tokenizer->sizeOfType(argument->typeStartToken());
+                if (argsize == 100) // unknown size
+                    argsize = 0;
+                while (Token::Match(tok2, "[ %num% ] [,)[]")) {
+                    argsize *= MathLib::toLongNumber(tok2->strAt(1));
+                    tok2 = tok2->tokAt(3);
+                }
+
+                MathLib::bigint arraysize = arrayInfo.element_size();
+                if (arraysize == 100) // unknown size
+                    arraysize = 0;
+                for (unsigned int i = 0; i < arrayInfo.num().size(); i++)
+                    arraysize *= arrayInfo.num(i);
+
+                if (Token::Match(tok2, "[,)]") && arraysize > 0 && argsize > arraysize)
+                    argumentSizeError(&tok, tok.str(), arrayInfo.varname());
+            }
+        }
+    }
 }
 
 
@@ -764,10 +822,10 @@ void CheckBufferOverrun::checkScopeForBody(const Token *tok, const ArrayInfo &ar
     if (!for3(tok2->next(), counter_varid, min_counter_value, max_counter_value, maxMinFlipped))
         return;
 
-    if (Token::Match(tok2->next(), "%var% =|+=|-=") && MathLib::toLongNumber(max_counter_value) <= size)
+    if (Token::Match(tok2->next(), "%var% =") && MathLib::toLongNumber(max_counter_value) < size)
         condition_out_of_bounds = false;
 
-    // Goto the end parenthesis of the for-statement: "for (x; y; z)" ..
+    // Goto the end parentheses of the for-statement: "for (x; y; z)" ..
     tok2 = tok->next()->link();
     if (!tok2 || !tok2->tokAt(5)) {
         bailout = true;
@@ -887,13 +945,17 @@ void CheckBufferOverrun::checkScope(const Token *tok, const std::vector<std::str
 
         // Array index..
         if ((varid > 0 && ((tok->str() == "return" || (!tok->isName() && !Token::Match(tok, "[.&]"))) && Token::Match(tok->next(), "%varid% [ %num% ]", varid))) ||
-            (varid == 0 && ((tok->str() == "return" || (!tok->isName() && !Token::Match(tok, "[.&]"))) && Token::Match(tok->next(), (varnames + " [ %num% ]").c_str())))) {
+            (varid == 0 && ((tok->str() == "return" || (!tok->isName() && !Token::Match(tok, "[.&]"))) && (Token::Match(tok->next(), (varnames + " [ %num% ]").c_str()) || Token::Match(tok->next(), (varname[0] +" [ %num% ] . " + varname[1] + " [ %num% ]").c_str()))))) {
             std::vector<MathLib::bigint> indexes;
             const Token *tok2 = tok->tokAt(2 + varc);
             for (; Token::Match(tok2, "[ %num% ]"); tok2 = tok2->tokAt(3)) {
                 const MathLib::bigint index = MathLib::toLongNumber(tok2->strAt(1));
                 indexes.push_back(index);
             }
+            for (; Token::Match(tok2->tokAt(3), "[ %num% ]"); tok2 = tok2->tokAt(3)) {
+                const MathLib::bigint index = MathLib::toLongNumber(tok2->strAt(4));
+                indexes.push_back(index);
+            }
 
             if (indexes.size() == arrayInfo.num().size()) {
                 // Check if the indexes point outside the whole array..
@@ -983,12 +1045,12 @@ void CheckBufferOverrun::checkScope(const Token *tok, const std::vector<std::str
             (varid == 0 && Token::Match(tok, ("strcpy|strcat ( " + varnames + " , %str% )").c_str()))) {
             const std::size_t len = Token::getStrLength(tok->tokAt(varc + 4));
             if (total_size > 0 && len >= (unsigned int)total_size) {
-                bufferOverrunError(tok, varid > 0 ? std::string("") : varnames);
+                bufferOverrunError(tok, varid > 0 ? std::string() : varnames);
                 continue;
             }
         } else if ((varid > 0 && Token::Match(tok, "strcpy|strcat ( %varid% , %var% )", varid)) ||
                    (varid == 0 && Token::Match(tok, ("strcpy|strcat ( " + varnames + " , %var% )").c_str()))) {
-            const Variable *var = _tokenizer->getSymbolDatabase()->getVariableFromVarId(tok->tokAt(4)->varId());
+            const Variable *var = tok->tokAt(4)->variable();
             if (var && var->isArray() && var->dimensions().size() == 1) {
                 const std::size_t len = (std::size_t)var->dimension(0);
                 if (total_size > 0 && len > (unsigned int)total_size) {
@@ -1162,7 +1224,7 @@ void CheckBufferOverrun::checkScope(const Token *tok, const ArrayInfo &arrayInfo
             }
         }
 
-        if ((Token::Match(tok, "strncpy|strncat ( %varid% , %var%", arrayInfo.varid()) && Token::Match(tok->linkAt(1)->tokAt(-2), ", %num% )"))) {
+        if ((Token::Match(tok, "strncpy|strncat ( %varid% ,", arrayInfo.varid()) && Token::Match(tok->linkAt(1)->tokAt(-2), ", %num% )"))) {
             const Token* param3 = tok->linkAt(1)->previous();
 
             // check for strncpy which is not terminated
@@ -1171,7 +1233,7 @@ void CheckBufferOverrun::checkScope(const Token *tok, const ArrayInfo &arrayInfo
                 unsigned int num = (unsigned int)MathLib::toLongNumber(param3->str());
 
                 // this is currently 'inconclusive'. See TestBufferOverrun::terminateStrncpy3
-                if (num >= total_size && _settings->isEnabled("style") && _settings->inconclusive) {
+                if (num >= total_size && _settings->isEnabled("warning") && _settings->inconclusive) {
                     const Token *tok2 = tok->next()->link()->next();
                     for (; tok2; tok2 = tok2->next()) {
                         if (tok2->varId() == tok->tokAt(2)->varId()) {
@@ -1254,6 +1316,29 @@ void CheckBufferOverrun::checkScope(const Token *tok, const ArrayInfo &arrayInfo
     }
 }
 
+//---------------------------------------------------------------------------
+// Checking member variables of structs..
+//---------------------------------------------------------------------------
+bool CheckBufferOverrun::isArrayOfStruct(const Token* tok, int &position)
+{
+    if (Token::Match(tok->next(), "%var% [ %num% ] ")) {
+        tok = tok->tokAt(4);
+        int i = 1;
+        while (true) {
+            if (Token::Match(tok->next(), "[ %num% ] ")) {
+                i++;
+                tok = tok->tokAt(4);
+            } else
+                break;
+        }
+        if (Token::Match(tok->next(),";")) {
+            position = i;
+            return true;
+        }
+    }
+    return false;
+}
+
 void CheckBufferOverrun::checkReadlinkBufferUsage(const Token* tok, const Token *scope_begin, const MathLib::bigint total_size, const bool is_readlinkat)
 {
     const Token* bufParam = tok->tokAt(2)->nextArgument();
@@ -1343,7 +1428,7 @@ void CheckBufferOverrun::checkGlobalAndLocalVariable()
             std::string type;
 
             // varid : The variable id for the array
-            unsigned int varid = 0;
+            const Variable *var = 0;
 
             // nextTok : number of tokens used in variable declaration - used to skip to next statement.
             int nextTok = 0;
@@ -1355,48 +1440,44 @@ void CheckBufferOverrun::checkGlobalAndLocalVariable()
             if (Token::Match(tok, "[*;{}] %var% = new %type% [ %num% ]")) {
                 size = MathLib::toLongNumber(tok->strAt(6));
                 type = tok->strAt(4);
-                varid = tok->next()->varId();
+                var = tok->next()->variable();
                 nextTok = 8;
             } else if (Token::Match(tok, "[*;{}] %var% = new %type% ( %num% )")) {
                 size = 1;
                 type = tok->strAt(4);
-                varid = tok->next()->varId();
+                var = tok->next()->variable();
                 nextTok = 8;
             } else if (Token::Match(tok, "[;{}] %var% = %str% ;") &&
                        tok->next()->varId() > 0 &&
                        NULL != Token::findmatch(_tokenizer->tokens(), "[;{}] const| %type% * %varid% ;", tok->next()->varId())) {
                 size = 1 + int(tok->tokAt(3)->strValue().size());
                 type = "char";
-                varid = tok->next()->varId();
+                var = tok->next()->variable();
                 nextTok = 4;
             } else if (Token::Match(tok, "[*;{}] %var% = malloc|alloca ( %num% ) ;")) {
                 size = MathLib::toLongNumber(tok->strAt(5));
                 type = "char";   // minimum type, typesize=1
-                varid = tok->next()->varId();
+                var = tok->next()->variable();
                 nextTok = 7;
 
-                if (varid > 0) {
-                    // get type of variable
-                    const Variable *var = _tokenizer->getSymbolDatabase()->getVariableFromVarId(varid);
-                    /** @todo false negatives: this may be too conservative */
-                    if (!var || var->typeEndToken()->str() != "*" || var->typeStartToken()->next() != var->typeEndToken())
-                        continue;
+                /** @todo false negatives: this may be too conservative */
+                if (!var || var->typeEndToken()->str() != "*" || var->typeStartToken()->next() != var->typeEndToken())
+                    continue;
 
-                    // get name of variable
-                    type = var->typeStartToken()->str();
+                // get name of variable
+                type = var->typeStartToken()->str();
 
-                    // malloc() gets count of bytes and not count of
-                    // elements, so we should calculate count of elements
-                    // manually
-                    unsigned int sizeOfType = _tokenizer->sizeOfType(var->typeStartToken());
-                    if (sizeOfType > 0)
-                        size /= static_cast<int>(sizeOfType);
-                }
+                // malloc() gets count of bytes and not count of
+                // elements, so we should calculate count of elements
+                // manually
+                unsigned int sizeOfType = _tokenizer->sizeOfType(var->typeStartToken());
+                if (sizeOfType > 0)
+                    size /= static_cast<int>(sizeOfType);
             } else {
                 continue;
             }
 
-            if (varid == 0)
+            if (var == 0)
                 continue;
 
             Token sizeTok(0);
@@ -1406,7 +1487,7 @@ void CheckBufferOverrun::checkGlobalAndLocalVariable()
                 continue;
 
             std::vector<std::string> v;
-            ArrayInfo temp(varid, tok->next()->str(), total_size / size, size);
+            ArrayInfo temp(var->varId(), tok->next()->str(), total_size / size, size);
             checkScope(tok->tokAt(nextTok), v, temp);
         }
     }
@@ -1430,7 +1511,6 @@ void CheckBufferOverrun::checkStructVariable()
         // check all variables to see if they are arrays
         std::list<Variable>::const_iterator var;
         for (var = scope->varlist.begin(); var != scope->varlist.end(); ++var) {
-            // find all array variables
             if (var->isArray()) {
                 // create ArrayInfo from the array variable
                 ArrayInfo arrayInfo(&*var, _tokenizer);
@@ -1472,10 +1552,24 @@ void CheckBufferOverrun::checkStructVariable()
                         if (tok3->str() != scope->className)
                             continue;
 
+                        // find all array variables
+                        int posOfSemicolon = -1;
+
                         // Declare variable: Fred fred1;
                         if (Token::Match(tok3->next(), "%var% ;"))
                             varname[0] = tok3->strAt(1);
 
+                        else if (isArrayOfStruct(tok3,posOfSemicolon)) {
+                            varname[0] = tok3->strAt(1);
+
+                            int pos = 2;
+                            for (int k = 0 ; k < posOfSemicolon; k++) {
+                                for (int index = pos; index < (pos + 3); index++)
+                                    tok3->strAt(index);
+                                pos += 3;
+                            }
+                        }
+
                         // Declare pointer or reference: Fred *fred1
                         else if (Token::Match(tok3->next(), "*|& %var% [,);=]"))
                             varname[0] = tok3->strAt(2);
@@ -1565,6 +1659,7 @@ void CheckBufferOverrun::checkStructVariable()
                         std::string varnames; // use class and member name for messages
                         for (unsigned int k = 0; k < varname.size(); ++k)
                             varnames += (k == 0 ? "" : ".") + varname[k];
+
                         temp.varname(varnames);
                         checkScope(CheckTok, varname, temp);
                     }
@@ -1857,7 +1952,7 @@ void CheckBufferOverrun::negativeIndex()
                 tok2 = tok2->previous()->link();
 
             if (tok2->previous() && tok2->previous()->varId()) {
-                const Variable *var = _tokenizer->getSymbolDatabase()->getVariableFromVarId(tok2->previous()->varId());
+                const Variable *var = tok2->previous()->variable();
                 if (var && var->isArray())
                     negativeIndexError(tok, index);
             }
@@ -2127,3 +2222,51 @@ void CheckBufferOverrun::arrayIndexThenCheckError(const Token *tok, const std::s
                 "Reorder conditions such as '(a[i] && i < 10)' to '(i < 10 && a[i])'. That way the array will "
                 "not be accessed if the index is out of limits.");
 }
+
+// -------------------------------------------------------------------------------------
+// Check the second and the third parameter of the POSIX function write and validate
+// their values.
+// The parameters have the following meaning:
+// - 1.parameter: file descripter (not required for this check)
+// - 2.parameter: is a null terminated character string of the content to write.
+// - 3.parameter: the number of bytes to write.
+//
+// This check is triggered if the size of the string ( 2. parameter) is lower than
+// the number of bytes provided at the 3. parameter.
+//
+// References:
+//  - http://gd.tuwien.ac.at/languages/c/programming-bbrown/c_075.htm
+//  - http://codewiki.wikidot.com/c:system-calls:write
+// -------------------------------------------------------------------------------------
+void CheckBufferOverrun::writeOutsideBufferSize()
+{
+    if (!_settings->standards.posix)
+        return;
+
+    const SymbolDatabase* symbolDatabase = _tokenizer->getSymbolDatabase();
+    const std::size_t functions = symbolDatabase->functionScopes.size();
+    for (std::size_t i = 0; i < functions; ++i) {
+        const Scope * scope = symbolDatabase->functionScopes[i];
+        for (const Token *tok = scope->classStart; tok && tok != scope->classEnd; tok = tok->next()) {
+            if (Token::Match(tok, "pwrite|write (") && Token::Match(tok->tokAt(2)->nextArgument(), "%str% , %num%")) {
+                const std::string & functionName(tok->str());
+                tok = tok->tokAt(4); // set tokenptr to %str% parameter
+                const std::size_t stringLength = Token::getStrLength(tok);
+                tok = tok->tokAt(2); // set tokenptr to %num% parameter
+                const MathLib::bigint writeLength = MathLib::toLongNumber(tok->str());
+                if (static_cast<unsigned long int>(writeLength) > stringLength)
+                    writeOutsideBufferSizeError(tok, stringLength, writeLength,functionName);
+            }
+        }
+    }
+}
+
+void CheckBufferOverrun::writeOutsideBufferSizeError(const Token *tok, const std::size_t stringLength, const MathLib::bigint writeLength, const std::string &strFunctionName)
+{
+    reportError(tok, Severity::error, "writeOutsideBufferSize",
+                "Writing '" +MathLib::longToString(writeLength-stringLength)+"' bytes outside buffer size.\n"
+                "The number of bytes to write ('" +MathLib::longToString(writeLength)+ "' bytes) are bigger than the source buffer ('" +MathLib::longToString(stringLength)+ "' bytes)."
+                " Please check the second and the third parameter of the function '"+strFunctionName+"'.");
+}
+// -------------------------------------------------------------------------------------
+// -------------------------------------------------------------------------------------
