@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -100,7 +100,7 @@ void CheckLeakAutoVar::deallocReturnError(const Token *tok, const std::string &v
 
 void CheckLeakAutoVar::configurationInfo(const Token* tok, const std::string &functionName)
 {
-    if (((!cfgalloc.empty() || !cfgdealloc.empty()) && _settings->isEnabled("style")) || _settings->experimental) {
+    if (((!cfgalloc.empty() || !cfgdealloc.empty()) && _settings->isEnabled("information")) || _settings->experimental) {
         reportError(tok,
                     Severity::information,
                     "leakconfiguration",
@@ -247,7 +247,7 @@ void CheckLeakAutoVar::checkScope(const Token * const startToken,
                     deallocUseError(tok, tok->str());
                 } else if (Token::simpleMatch(tok->tokAt(-2), "= &")) {
                     varInfo->erase(tok->varId());
-                } else if (Token::simpleMatch(tok->previous(), "=")) {
+                } else if (tok->strAt(-1) == "=") {
                     varInfo->erase(tok->varId());
                 }
             } else if (Token::Match(tok->previous(), "& %var% = %var% ;")) {
@@ -311,7 +311,7 @@ void CheckLeakAutoVar::checkScope(const Token * const startToken,
             varInfo->erase(tok->varId());
 
             // not a local variable nor argument?
-            const Variable *var = _tokenizer->getSymbolDatabase()->getVariableFromVarId(tok->varId());
+            const Variable *var = tok->variable();
             if (var && !var->isArgument() && !var->isLocal()) {
                 continue;
             }
