@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -59,8 +59,10 @@ public:
         checkOther.strPlusChar();
         checkOther.sizeofsizeof();
         checkOther.sizeofCalculation();
+        checkOther.suspiciousSizeofCalculation();
         checkOther.checkRedundantAssignment();
         checkOther.checkRedundantAssignmentInSwitch();
+        checkOther.checkSuspiciousCaseInSwitch();
         checkOther.checkAssignmentInAssert();
         checkOther.checkSizeofForArrayParameter();
         checkOther.checkSizeofForPointerSize();
@@ -74,9 +76,11 @@ public:
         checkOther.checkVariableScope();
         checkOther.clarifyCondition();   // not simplified because ifAssign
         checkOther.checkComparisonOfBoolExpressionWithInt();
+        checkOther.checkComparisonOfBoolWithInt();
         checkOther.checkSignOfUnsignedVariable();  // don't ignore casts (#3574)
         checkOther.checkIncompleteArrayFill();
         checkOther.checkSuspiciousStringCompare();
+        checkOther.checkVarFuncNullUB();
     }
 
     /** @brief Run checks against the simplified token list */
@@ -89,12 +93,14 @@ public:
         checkOther.clarifyStatement();
         checkOther.checkConstantFunctionParameter();
         checkOther.checkIncompleteStatement();
+        checkOther.checkCastIntToCharAndBack();
 
         checkOther.invalidFunctionUsage();
         checkOther.checkZeroDivision();
         checkOther.checkMathFunctions();
         checkOther.checkCCTypeFunctions();
 
+        checkOther.redundantGetAndSetUserId();
         checkOther.checkIncorrectLogicOperator();
         checkOther.checkMisusedScopedObject();
         checkOther.checkComparisonOfFuncReturningBool();
@@ -102,10 +108,10 @@ public:
         checkOther.checkMemsetZeroBytes();
         checkOther.checkIncorrectStringCompare();
         checkOther.checkIncrementBoolean();
-        checkOther.checkComparisonOfBoolWithInt();
         checkOther.checkSwitchCaseFallThrough();
         checkOther.checkAlwaysTrueOrFalseStringCompare();
         checkOther.checkModuloAlwaysTrueFalse();
+        checkOther.checkPipeParameterSize();
 
         checkOther.checkAssignBoolToPointer();
         checkOther.checkBitwiseOnBoolean();
@@ -113,9 +119,11 @@ public:
         checkOther.checkDoubleFree();
         checkOther.checkRedundantCopy();
         checkOther.checkNegativeBitwiseShift();
+        checkOther.checkSuspiciousEqualityComparison();
+        checkOther.checkSleepTimeInterval();
     }
 
-    /** To check the dead code in a program, which is unaccessible due to the counter-conditions check in nested-if statements **/
+    /** To check the dead code in a program, which is inaccessible due to the counter-conditions check in nested-if statements **/
     void oppositeInnerCondition();
 
     /** @brief Clarify calculation for ".. a * b ? .." */
@@ -147,7 +155,7 @@ public:
 
     /** @brief %Check scope of variables */
     void checkVariableScope();
-    void lookupVar(const Token *tok, const Variable* var);
+    static bool checkInnerScope(const Token *tok, const Variable* var, bool& used);
 
     /** @brief %Check for constant function parameter */
     void checkConstantFunctionParameter();
@@ -176,12 +184,24 @@ public:
     /** @brief %Check for calculations inside sizeof */
     void sizeofCalculation();
 
-    /** @brief copying to memory or assigning to a variablen twice */
+    /** @brief % Check for seteuid(geteuid()) or setuid(getuid())*/
+    void redundantGetAndSetUserId();
+
+    /** @brief %Check for suspicious calculations with sizeof results */
+    void suspiciousSizeofCalculation();
+
+    /** @brief copying to memory or assigning to a variable twice */
     void checkRedundantAssignment();
 
     /** @brief %Check for assigning to the same variable twice in a switch statement*/
     void checkRedundantAssignmentInSwitch();
 
+    /** @brief %Check for code like 'case A||B:'*/
+    void checkSuspiciousCaseInSwitch();
+
+    /** @brief %Check for code like 'case A||B:'*/
+    void checkSuspiciousEqualityComparison();
+
     /** @brief %Check for switch case fall through without comment */
     void checkSwitchCaseFallThrough();
 
@@ -262,7 +282,7 @@ public:
 
     /** @brief %Check for free() operations on invalid memory locations */
     void checkInvalidFree();
-    void invalidFreeError(const Token *tok);
+    void invalidFreeError(const Token *tok, bool inconclusive);
 
     /** @brief %Check for double free or double close operations */
     void checkDoubleFree();
@@ -277,14 +297,35 @@ public:
     /** @brief %Check for buffers that are filled incompletely with memset and similar functions */
     void checkIncompleteArrayFill();
 
+    /** @brief %Check that variadic function calls don't use NULL. If NULL is \#defined as 0 and the function expects a pointer, the behaviour is undefined. */
+    void checkVarFuncNullUB();
+
+    /** @brief %Check that calling the POSIX pipe() system call is called with an integer array of size two. */
+    void checkPipeParameterSize();
+
+    /** @brief %Check to avoid casting a return value to unsigned char and then back to integer type.  */
+    void checkCastIntToCharAndBack();
+
+    /** @brief %Check providing too big sleep time intervals on POSIX systems. */
+    void checkSleepTimeInterval();
+
 private:
+    bool isUnsigned(const Variable *var) const;
+    static bool isSigned(const Variable *var);
+
     // Error messages..
+    void checkSleepTimeError(const Token *tok, const std::string &strDim);
+    void checkCastIntToCharAndBackError(const Token *tok, const std::string &strFunctionName);
+    void checkPipeParameterSizeError(const Token *tok, const std::string &strVarName, const std::string &strDim);
     void oppositeInnerConditionError(const Token *tok);
     void clarifyCalculationError(const Token *tok, const std::string &op);
     void clarifyConditionError(const Token *tok, bool assign, bool boolop);
     void clarifyStatementError(const Token* tok);
     void sizeofsizeofError(const Token *tok);
     void sizeofCalculationError(const Token *tok, bool inconclusive);
+    void redundantGetAndSetUserIdError(const Token *tok);
+    void multiplySizeofError(const Token *tok);
+    void divideSizeofError(const Token *tok);
     void cstyleCastError(const Token *tok);
     void invalidPointerCastError(const Token* tok, const std::string& from, const std::string& to, bool inconclusive);
     void dangerousUsageStrtolError(const Token *tok, const std::string& funcname);
@@ -299,12 +340,14 @@ private:
     void zerodivError(const Token *tok);
     void mathfunctionCallError(const Token *tok, const unsigned int numParam = 1);
     void cctypefunctionCallError(const Token *tok, const std::string &functionName, const std::string &value);
-    void redundantAssignmentError(const Token *tok1, const Token* tok2, const std::string& var);
+    void redundantAssignmentError(const Token *tok1, const Token* tok2, const std::string& var, bool inconclusive);
     void redundantAssignmentInSwitchError(const Token *tok1, const Token *tok2, const std::string &var);
     void redundantCopyError(const Token *tok1, const Token* tok2, const std::string& var);
     void redundantCopyInSwitchError(const Token *tok1, const Token* tok2, const std::string &var);
     void redundantBitwiseOperationInSwitchError(const Token *tok, const std::string &varname);
     void switchCaseFallThrough(const Token *tok);
+    void suspiciousCaseInSwitchError(const Token* tok, const std::string& operatorString);
+    void suspiciousEqualityComparisonError(const Token* tok);
     void selfAssignmentError(const Token *tok, const std::string &varname);
     void assignmentInAssertError(const Token *tok, const std::string &varname);
     void incorrectLogicOperatorError(const Token *tok, const std::string &condition, bool always);
@@ -343,6 +386,7 @@ private:
     void negativeBitwiseShiftError(const Token *tok);
     void redundantCopyError(const Token *tok, const std::string &varname);
     void incompleteArrayFillError(const Token* tok, const std::string& buffer, const std::string& function, bool boolean);
+    void varFuncNullUBError(const Token *tok);
 
     void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings) const {
         CheckOther c(0, settings, errorLogger);
@@ -363,13 +407,16 @@ private:
         c.doubleFreeError(0, "varname");
         c.invalidPointerCastError(0, "float", "double", false);
         c.negativeBitwiseShiftError(0);
+        c.checkPipeParameterSizeError(0, "varname", "dimension");
+        c.checkSleepTimeError(0,"dimension");
 
         //performance
         c.redundantCopyError(0, "varname");
         c.redundantCopyError(0, 0, "var");
-        c.redundantAssignmentError(0, 0, "var");
+        c.redundantAssignmentError(0, 0, "var", false);
 
         // style/warning
+        c.checkCastIntToCharAndBackError(0,"func_name");
         c.oppositeInnerConditionError(0);
         c.cstyleCastError(0);
         c.dangerousUsageStrtolError(0, "strtol");
@@ -381,9 +428,13 @@ private:
         c.strPlusCharError(0);
         c.sizeofsizeofError(0);
         c.sizeofCalculationError(0, false);
+        c.multiplySizeofError(0);
+        c.divideSizeofError(0);
         c.redundantAssignmentInSwitchError(0, 0, "var");
         c.redundantCopyInSwitchError(0, 0, "var");
         c.switchCaseFallThrough(0);
+        c.suspiciousCaseInSwitchError(0, "||");
+        c.suspiciousEqualityComparisonError(0);
         c.selfAssignmentError(0, "varname");
         c.assignmentInAssertError(0, "varname");
         c.incorrectLogicOperatorError(0, "foo > 3 && foo < 4", true);
@@ -414,6 +465,7 @@ private:
         c.cctypefunctionCallError(0, "funname", "value");
         c.moduloAlwaysTrueFalseError(0, "1");
         c.incompleteArrayFillError(0, "buffer", "memset", false);
+        c.varFuncNullUBError(0);
     }
 
     static std::string myName() {
@@ -435,13 +487,16 @@ private:
                "* free() or delete of an invalid memory location\n"
                "* double free() or double closedir()\n"
                "* bitwise operation with negative right operand\n"
+               "* provide wrong dimensioned array to pipe() system command (--std=posix)\n"
+               "* cast the return values of getc(),fgetc() and getchar() to character and compare it to EOF\n"
+               "* provide too big sleep times on POSIX systems\n"
 
                //performance
                "* redundant data copying for const variable\n"
                "* subsequent assignment or copying to a variable or buffer\n"
 
                // style
-               "* Find dead code which is unaccessible due to the counter-conditions check in nested if statements\n"
+               "* Find dead code which is inaccessible due to the counter-conditions check in nested if statements\n"
                "* C-style pointer cast in cpp file\n"
                "* casting between incompatible pointer types\n"
                "* redundant if\n"
@@ -452,14 +507,17 @@ private:
                "* [[charvar|check how signed char variables are used]]\n"
                "* variable scope can be limited\n"
                "* condition that is always true/false\n"
-               "* unusal pointer arithmetic. For example: \"abc\" + 'd'\n"
+               "* unusual pointer arithmetic. For example: \"abc\" + 'd'\n"
                "* redundant assignment in a switch statement\n"
                "* redundant pre/post operation in a switch statement\n"
                "* redundant bitwise operation in a switch statement\n"
                "* redundant strcpy in a switch statement\n"
                "* look for 'sizeof sizeof ..'\n"
                "* look for calculations inside sizeof()\n"
+               "* look for suspicious calculations with sizeof()\n"
                "* assignment of a variable to itself\n"
+               "* Suspicious case labels in switch()\n"
+               "* Suspicious equality comparisons\n"
                "* mutual exclusion over || always evaluating to true\n"
                "* Clarify calculation with parentheses\n"
                "* using increment on boolean\n"
@@ -479,7 +537,9 @@ private:
                "* Suspicious use of ; at the end of 'if/for/while' statement.\n"
                "* incorrect usage of functions from ctype library.\n"
                "* Comparisons of modulo results that are always true/false.\n"
-               "* Array filled incompletely using memset/memcpy/memmove.\n";
+               "* Array filled incompletely using memset/memcpy/memmove.\n"
+               "* redundant get and set function of user id (--std=posix).\n"
+               "* Passing NULL pointer to function with variable number of arguments leads to UB on some platforms.\n";
     }
 
     void checkExpressionRange(const std::list<const Function*> &constFunctions,
