@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,8 @@
 #include "checknullpointer.h"   // CheckNullPointer::parseFunctionCall
 #include "symboldatabase.h"
 #include <algorithm>
+#include <map>
+#include <cassert>
 //---------------------------------------------------------------------------
 
 // Register this check class (by creating a static instance of it)
@@ -275,9 +277,12 @@ private:
 
                 CheckUninitVar *checkUninitVar = dynamic_cast<CheckUninitVar *>(c->owner);
                 if (checkUninitVar) {
-                    if (c->strncpy_ || c->memset_nonzero)
+                    if (c->strncpy_ || c->memset_nonzero) {
+                        if (!Token::Match(c->var->typeStartToken(), "char|wchar_t")) {
+                            continue;
+                        }
                         checkUninitVar->uninitstringError(tok, c->var->name(), c->strncpy_);
-                    else if (c->var->isPointer() && !c->var->isArray() && c->alloc)
+                    } else if (c->var->isPointer() && !c->var->isArray() && c->alloc)
                         checkUninitVar->uninitdataError(tok, c->var->name());
                     else
                         checkUninitVar->uninitvarError(tok, c->var->name());
@@ -365,7 +370,7 @@ private:
             if (tok2->varId() &&
                 !Token::Match(tok2->previous(), "&|::") &&
                 !Token::simpleMatch(tok2->tokAt(-2), "& (") &&
-                !Token::simpleMatch(tok2->next(), "=")) {
+                tok2->strAt(1) != "=") {
                 // Multiple assignments..
                 if (Token::Match(tok2->next(), ".|[")) {
                     const Token * tok3 = tok2;
@@ -399,7 +404,7 @@ private:
     const Token *parse(const Token &tok, std::list<ExecutionPath *> &checks) const {
         // Variable declaration..
         if (tok.varId() && Token::Match(&tok, "%var% [[;]")) {
-            const Variable* var2 = symbolDatabase->getVariableFromVarId(tok.varId());
+            const Variable* var2 = tok.variable();
             if (var2 && var2->nameToken() == &tok && !var2->isStatic() && !var2->isExtern() && !var2->isConst()) {
                 if (tok.linkAt(1)) { // array
                     const Token* endtok = tok.next();
@@ -481,7 +486,7 @@ private:
             }
 
             // Used..
-            if (Token::Match(tok.previous(), "[[(,+-*/|=] %var% ]|)|,|;|%op%")) {
+            if (Token::Match(tok.previous(), "[[(,+-*/|=] %var% ]|)|,|;|%op%") && !tok.next()->isAssignmentOp()) {
                 // Taking address of array..
                 std::list<ExecutionPath *>::const_iterator it;
                 for (it = checks.begin(); it != checks.end(); ++it) {
@@ -542,12 +547,12 @@ private:
                 }
             }
 
-            if (Token::simpleMatch(tok.next(), "(")) {
+            if (tok.strAt(1) == "(") {
                 use_pointer(checks, &tok);
             }
 
             if (Token::Match(tok.tokAt(-2), "[;{}] *")) {
-                if (Token::simpleMatch(tok.next(), "=")) {
+                if (tok.strAt(1) == "=") {
                     // is the pointer used in the rhs?
                     bool used = false;
                     for (const Token *tok2 = tok.tokAt(2); tok2; tok2 = tok2->next()) {
@@ -574,22 +579,23 @@ private:
                     return tok.tokAt(3);
             }
 
-            else if ((!isC && Token::Match(tok.previous(), "<<|>>")) || Token::simpleMatch(tok.next(), "=")) {
+            else if ((!isC && (Token::Match(tok.previous(), "<<|>>") || Token::Match(tok.previous(), "[;{}] %var% <<"))) ||
+                     tok.strAt(1) == "=") {
                 // TODO: Don't bail out for "<<" and ">>" if these are
                 // just computations
                 ExecutionPath::bailOutVar(checks, tok.varId());
                 return &tok;
             }
 
-            if (Token::simpleMatch(tok.next(), "[")) {
+            if (tok.strAt(1) == "[" && tok.next()->link()) {
                 const Token *tok2 = tok.next()->link();
-                if (Token::simpleMatch(tok2 ? tok2->next() : 0, "=")) {
+                if (tok2->strAt(1) == "=") {
                     ExecutionPath::bailOutVar(checks, tok.varId());
                     return &tok;
                 }
             }
 
-            if (Token::simpleMatch(tok.previous(), "delete") ||
+            if (tok.strAt(-1) == "delete" ||
                 Token::simpleMatch(tok.tokAt(-3), "delete [ ]")) {
                 dealloc_pointer(checks, &tok);
                 return &tok;
@@ -758,7 +764,7 @@ private:
         }
 
         if (tok.varId()) {
-            if (Token::simpleMatch(tok.previous(), "=")) {
+            if (tok.strAt(-1) == "=") {
                 if (Token::Match(tok.tokAt(-3), "& %var% =")) {
                     bailOutVar(checks, tok.varId());
                     return &tok;
@@ -780,12 +786,12 @@ private:
                 }
             }
 
-            if (Token::simpleMatch(tok.next(), ".")) {
+            if (tok.strAt(1) == ".") {
                 bailOutVar(checks, tok.varId());
                 return &tok;
             }
 
-            if (Token::simpleMatch(tok.next(), "[")) {
+            if (tok.strAt(1) == "[") {
                 ExecutionPath::bailOutVar(checks, tok.varId());
                 return &tok;
             }
@@ -795,7 +801,7 @@ private:
                 return &tok;
             }
 
-            if (Token::simpleMatch(tok.previous(), "&")) {
+            if (tok.strAt(-1) == "&") {
                 ExecutionPath::bailOutVar(checks, tok.varId());
             }
         }
@@ -1013,7 +1019,7 @@ std::set<std::string> UninitVar::uvarFunctions;
 /// @}
 
 
-void CheckUninitVar::analyse(const Token * const tokens, std::set<std::string> &func) const
+void CheckUninitVar::analyse(const Token * tokens, std::set<std::string> &func) const
 {
     UninitVar::analyseFunctions(tokens, func);
 }
@@ -1040,13 +1046,12 @@ void CheckUninitVar::executionPaths()
 void CheckUninitVar::check()
 {
     const SymbolDatabase *symbolDatabase = _tokenizer->getSymbolDatabase();
-    std::list<Scope>::const_iterator func_scope;
+    std::list<Scope>::const_iterator scope;
 
-    // scan every function
-    for (func_scope = symbolDatabase->scopeList.begin(); func_scope != symbolDatabase->scopeList.end(); ++func_scope) {
-        // only check functions
-        if (func_scope->type == Scope::eFunction) {
-            checkScope(&*func_scope);
+    // check every executable scope
+    for (scope = symbolDatabase->scopeList.begin(); scope != symbolDatabase->scopeList.end(); ++scope) {
+        if (scope->isExecutable()) {
+            checkScope(&*scope);
         }
     }
 }
@@ -1054,7 +1059,8 @@ void CheckUninitVar::check()
 void CheckUninitVar::checkScope(const Scope* scope)
 {
     for (std::list<Variable>::const_iterator i = scope->varlist.begin(); i != scope->varlist.end(); ++i) {
-        if ((i->type() && !i->isPointer()) || i->isStatic() || i->isExtern() || i->isConst() || i->isArray() || i->isReference())
+        if ((_tokenizer->isCPP() && i->type() && !i->isPointer() && i->type()->needInitialization != Type::True) ||
+            i->isStatic() || i->isExtern() || i->isConst() || i->isArray() || i->isReference())
             continue;
         if (i->nameToken()->strAt(1) == "(")
             continue;
@@ -1064,7 +1070,7 @@ void CheckUninitVar::checkScope(const Scope* scope)
             if (tok->str() == "(") {
                 forHead = true;
                 break;
-            } else if (tok->str() == "{" || tok->str() == ";" || tok->str() == "}")
+            } else if (Token::Match(tok, "[{};]"))
                 break;
         }
         if (forHead)
@@ -1079,28 +1085,77 @@ void CheckUninitVar::checkScope(const Scope* scope)
         while (tok && tok->str() != ";")
             tok = tok->next();
         if (stdtype || i->isPointer())
-            checkScopeForVariable(tok, *i, NULL);
-    }
-
-    for (std::list<Scope*>::const_iterator i = scope->nestedList.begin(); i != scope->nestedList.end(); ++i) {
-        if (!(*i)->isClassOrStruct())
-            checkScope(*i);
+            checkScopeForVariable(scope, tok, *i, NULL, NULL, "");
+        if (Token::Match(i->typeStartToken(), "struct %type% %var% ;")) {
+            const std::string structname(i->typeStartToken()->next()->str());
+            const SymbolDatabase * symbolDatabase = _tokenizer->getSymbolDatabase();
+            for (std::size_t j = 0U; j < symbolDatabase->classAndStructScopes.size(); ++j) {
+                const Scope *scope2 = symbolDatabase->classAndStructScopes[j];
+                if (scope2->className == structname && scope2->numConstructors == 0U) {
+                    for (std::list<Variable>::const_iterator it = scope2->varlist.begin(); it != scope2->varlist.end(); ++it) {
+                        const Variable &var = *it;
+                        if (!var.isArray())
+                            checkScopeForVariable(scope, tok, *i, NULL, NULL, var.name());
+                    }
+                }
+            }
+        }
     }
 }
 
-bool CheckUninitVar::checkScopeForVariable(const Token *tok, const Variable& var, bool * const possibleInit)
+static void conditionAlwaysTrueOrFalse(const Token *tok, const std::map<unsigned int, int> &variableValue, bool *alwaysTrue, bool *alwaysFalse)
+{
+    assert(Token::simpleMatch(tok, "if ("));
+
+    const Token *vartok = tok->tokAt(2);
+    const bool NOT(vartok->str() == "!");
+    if (NOT)
+        vartok = vartok->next();
+
+    while (Token::Match(vartok, "%var% . %var%"))
+        vartok = vartok->tokAt(2);
+
+    std::map<unsigned int, int>::const_iterator it = variableValue.find(vartok->varId());
+    if (it == variableValue.end())
+        return;
+
+    // always true
+    if (Token::Match(vartok, "%var% %oror%|)")) {
+        if (NOT)
+            *alwaysTrue = bool(it->second == 0);
+        else
+            *alwaysTrue = bool(it->second != 0);
+    } else if (Token::Match(vartok, "%var% == %num% %or%|)")) {
+        *alwaysTrue = bool(it->second == MathLib::toLongNumber(vartok->strAt(2)));
+    } else if (Token::Match(vartok, "%var% != %num% %or%|)")) {
+        *alwaysTrue = bool(it->second != MathLib::toLongNumber(vartok->strAt(2)));
+    }
+
+    // always false
+    if (Token::Match(vartok, "%var% &&|)")) {
+        if (NOT)
+            *alwaysFalse = bool(it->second != 0);
+        else
+            *alwaysFalse = bool(it->second == 0);
+    } else if (Token::Match(vartok, "%var% == %num% &&|)")) {
+        *alwaysFalse = bool(it->second != MathLib::toLongNumber(vartok->strAt(2)));
+    } else if (Token::Match(vartok, "%var% != %num% &&|)")) {
+        *alwaysFalse = bool(it->second == MathLib::toLongNumber(vartok->strAt(2)));
+    }
+}
+
+bool CheckUninitVar::checkScopeForVariable(const Scope* scope, const Token *tok, const Variable& var, bool * const possibleInit, bool * const noreturn, const std::string &membervar)
 {
     const bool suppressErrors(possibleInit && *possibleInit);
 
     if (possibleInit)
         *possibleInit = false;
 
-    bool ret = false;
-
     unsigned int number_of_if = 0;
 
-    // variables that are known to be non-zero
-    std::set<unsigned int> notzero;
+    // variable values
+    std::map<unsigned int, int> variableValue;
+    static const int NOT_ZERO = (1<<30); // special variable value
 
     for (; tok; tok = tok->next()) {
         // End of scope..
@@ -1109,15 +1164,18 @@ bool CheckUninitVar::checkScopeForVariable(const Token *tok, const Variable& var
                 *possibleInit = true;
 
             // might be a noreturn function..
-            if (_tokenizer->IsScopeNoReturn(tok))
+            if (_tokenizer->IsScopeNoReturn(tok)) {
+                if (noreturn)
+                    *noreturn = true;
                 return true;
+            }
 
             break;
         }
 
         // Unconditional inner scope..
         if (tok->str() == "{" && Token::Match(tok->previous(), "[;{}]")) {
-            if (checkScopeForVariable(tok->next(), var, possibleInit))
+            if (checkScopeForVariable(scope, tok->next(), var, possibleInit, NULL, membervar))
                 return true;
             tok = tok->link();
             continue;
@@ -1125,17 +1183,30 @@ bool CheckUninitVar::checkScopeForVariable(const Token *tok, const Variable& var
 
         // assignment with nonzero constant..
         if (Token::Match(tok->previous(), "[;{}] %var% = - %var% ;") && tok->varId() > 0)
-            notzero.insert(tok->varId());
+            variableValue[tok->varId()] = NOT_ZERO;
 
         // Inner scope..
         if (Token::simpleMatch(tok, "if (")) {
+            bool alwaysTrue = false;
+            bool alwaysFalse = false;
+
+            conditionAlwaysTrueOrFalse(tok, variableValue, &alwaysTrue, &alwaysFalse);
+
             // initialization / usage in condition..
-            if (checkIfForWhileHead(tok->next(), var, suppressErrors, bool(number_of_if == 0)))
+            if (!alwaysTrue && checkIfForWhileHead(tok->next(), var, suppressErrors, bool(number_of_if == 0), membervar))
                 return true;
 
             // checking if a not-zero variable is zero => bail out
-            if (Token::Match(tok, "if ( %var% )") && notzero.find(tok->tokAt(2)->varId()) != notzero.end())
-                return true;   // this scope is not fully analysed => return true
+            unsigned int condVarId = 0, condVarValue = 0;
+            if (Token::Match(tok, "if ( %var% )")) {
+                std::map<unsigned int,int>::const_iterator it = variableValue.find(tok->tokAt(2)->varId());
+                if (it != variableValue.end() && it->second == NOT_ZERO)
+                    return true;   // this scope is not fully analysed => return true
+                else {
+                    condVarId = tok->tokAt(2)->varId();
+                    condVarValue = NOT_ZERO;
+                }
+            }
 
             // goto the {
             tok = tok->next()->link()->next();
@@ -1144,7 +1215,38 @@ bool CheckUninitVar::checkScopeForVariable(const Token *tok, const Variable& var
                 break;
             if (tok->str() == "{") {
                 bool possibleInitIf(number_of_if > 0 || suppressErrors);
-                const bool initif = checkScopeForVariable(tok->next(), var, &possibleInitIf);
+                bool noreturnIf = false;
+                const bool initif = !alwaysFalse && checkScopeForVariable(scope, tok->next(), var, &possibleInitIf, &noreturnIf, membervar);
+
+                // bail out for such code:
+                //    if (a) x=0;    // conditional initialization
+                //    if (b) return; // cppcheck doesn't know if b can be false when a is false.
+                //    x++;           // it's possible x is always initialized
+                if (!alwaysTrue && noreturnIf && number_of_if > 0) {
+                    if (_settings->debugwarnings) {
+                        std::string condition;
+                        for (const Token *tok2 = tok->linkAt(-1); tok2 != tok; tok2 = tok2->next()) {
+                            condition += tok2->str();
+                            if (tok2->isName() && tok2->next()->isName())
+                                condition += ' ';
+                        }
+                        reportError(tok, Severity::debug, "debug", "uninitialized variables: bailout. can't determine if this condition can be false when previous condition is false: " + condition);
+                    }
+                    return true;
+                }
+
+                std::map<unsigned int, int> varValueIf;
+                if (!initif && !noreturnIf) {
+                    for (const Token *tok2 = tok; tok2 && tok2 != tok->link(); tok2 = tok2->next()) {
+                        if (Token::Match(tok2, "[;{}.] %var% = - %var% ;"))
+                            varValueIf[tok2->next()->varId()] = NOT_ZERO;
+                        if (Token::Match(tok2, "[;{}.] %var% = %num% ;"))
+                            varValueIf[tok2->next()->varId()] = (int)MathLib::toLongNumber(tok2->strAt(3));
+                    }
+                }
+
+                if (initif && condVarId > 0U)
+                    variableValue[condVarId] = condVarValue ^ NOT_ZERO;
 
                 // goto the }
                 tok = tok->link();
@@ -1160,7 +1262,22 @@ bool CheckUninitVar::checkScopeForVariable(const Token *tok, const Variable& var
                     tok = tok->tokAt(2);
 
                     bool possibleInitElse(number_of_if > 0 || suppressErrors);
-                    const bool initelse = checkScopeForVariable(tok->next(), var, &possibleInitElse);
+                    bool noreturnElse = false;
+                    const bool initelse = checkScopeForVariable(scope, tok->next(), var, &possibleInitElse, NULL, membervar);
+
+                    std::map<unsigned int, int> varValueElse;
+                    if (!initelse && !noreturnElse) {
+                        for (const Token *tok2 = tok; tok2 && tok2 != tok->link(); tok2 = tok2->next()) {
+                            if (Token::Match(tok2, "[;{}.] %var% = - %var% ;"))
+                                varValueElse[tok2->next()->varId()] = NOT_ZERO;
+                            if (Token::Match(tok2, "[;{}.] %var% = %num% ;"))
+                                varValueElse[tok2->next()->varId()] = (int)MathLib::toLongNumber(tok2->strAt(3));
+                        }
+                    }
+
+
+                    if (initelse && condVarId > 0U && !noreturnIf && !noreturnElse)
+                        variableValue[condVarId] = condVarValue;
 
                     // goto the }
                     tok = tok->link();
@@ -1168,8 +1285,14 @@ bool CheckUninitVar::checkScopeForVariable(const Token *tok, const Variable& var
                     if (initif && initelse)
                         return true;
 
-                    if (initif || initelse || possibleInitElse)
+                    if ((initif && noreturnElse) || (initelse && noreturnIf))
+                        return true;
+
+                    if ((initif || initelse || possibleInitElse) && !noreturnIf && !noreturnElse) {
                         ++number_of_if;
+                        variableValue.insert(varValueIf.begin(), varValueIf.end());
+                        variableValue.insert(varValueElse.begin(), varValueElse.end());
+                    }
                 }
             }
         }
@@ -1192,18 +1315,18 @@ bool CheckUninitVar::checkScopeForVariable(const Token *tok, const Variable& var
         if (Token::Match(tok, "sizeof|typeof|offsetof|decltype ("))
             tok = tok->next()->link();
 
-        // for..
-        if (Token::simpleMatch(tok, "for (")) {
+        // for/while..
+        if (Token::Match(tok, "for|while (")) {
             // is variable initialized in for-head (don't report errors yet)?
-            if (checkIfForWhileHead(tok->next(), var, true, false))
+            if (checkIfForWhileHead(tok->next(), var, true, false, membervar))
                 return true;
 
             // goto the {
             const Token *tok2 = tok->next()->link()->next();
 
             if (tok2 && tok2->str() == "{") {
-                bool possibleinit = true;
-                bool init = checkScopeForVariable(tok2->next(), var, &possibleinit);
+                bool possibleinit = false;
+                bool init = checkLoopBody(tok2, var, membervar, suppressErrors);
 
                 // variable is initialized in the loop..
                 if (possibleinit || init)
@@ -1211,8 +1334,11 @@ bool CheckUninitVar::checkScopeForVariable(const Token *tok, const Variable& var
 
                 // is variable used in for-head?
                 if (!suppressErrors) {
-                    checkIfForWhileHead(tok->next(), var, false, bool(number_of_if == 0));
+                    checkIfForWhileHead(tok->next(), var, false, bool(number_of_if == 0), membervar);
                 }
+
+                // goto "}"
+                tok = tok2->link();
             }
         }
 
@@ -1226,32 +1352,79 @@ bool CheckUninitVar::checkScopeForVariable(const Token *tok, const Variable& var
             return true;
         }
 
-        if (Token::Match(tok, "return|break|continue|throw|goto"))
-            ret = true;
-        else if (ret && tok->str() == ";")
-            return true;
+        if (Token::Match(tok, "return|break|continue|throw|goto")) {
+            if (noreturn)
+                *noreturn = true;
+
+            while (tok && tok->str() != ";") {
+                // variable is seen..
+                if (tok->varId() == var.varId()) {
+                    if (!membervar.empty()) {
+                        if (Token::Match(tok, "%var% . %var% ;|%cop%") && tok->strAt(2) == membervar)
+                            uninitStructMemberError(tok, tok->str() + "." + membervar);
+                        else
+                            return true;
+                    }
+
+                    // Use variable
+                    else if (!suppressErrors && isVariableUsage(tok, var.isPointer(), _tokenizer->isCPP()))
+                        uninitvarError(tok, tok->str());
+
+                    else
+                        // assume that variable is assigned
+                        return true;
+                }
+
+                else if (Token::Match(tok, "sizeof|typeof|offsetof|decltype ("))
+                    tok = tok->linkAt(1);
+
+                tok = tok->next();
+            }
+
+            return bool(noreturn==NULL);
+        }
 
         // variable is seen..
         if (tok->varId() == var.varId()) {
-            // Use variable
-            if (!suppressErrors && isVariableUsage(tok, var.isPointer()))
-                uninitvarError(tok, tok->str());
+            if (!membervar.empty()) {
+                if (isMemberVariableAssignment(tok, membervar))
+                    return true;
 
-            else
-                // assume that variable is assigned
-                return true;
+                if (isMemberVariableUsage(tok, var.isPointer(), membervar))
+                    uninitStructMemberError(tok, tok->str() + "." + membervar);
+
+            } else {
+                // Use variable
+                if (!suppressErrors && isVariableUsage(tok, var.isPointer(), _tokenizer->isCPP()))
+                    uninitvarError(tok, tok->str());
+
+                else
+                    // assume that variable is assigned
+                    return true;
+            }
         }
     }
 
-    return ret;
+    return false;
 }
 
-bool CheckUninitVar::checkIfForWhileHead(const Token *startparanthesis, const Variable& var, bool suppressErrors, bool isuninit)
+bool CheckUninitVar::checkIfForWhileHead(const Token *startparentheses, const Variable& var, bool suppressErrors, bool isuninit, const std::string &membervar)
 {
-    const Token * const endpar = startparanthesis->link();
-    for (const Token *tok = startparanthesis->next(); tok && tok != endpar; tok = tok->next()) {
+    const Token * const endpar = startparentheses->link();
+    for (const Token *tok = startparentheses->next(); tok && tok != endpar; tok = tok->next()) {
         if (tok->varId() == var.varId()) {
-            if (isVariableUsage(tok, var.isPointer())) {
+            if (Token::Match(tok, "%var% . %var%")) {
+                if (tok->strAt(2) == membervar) {
+                    if (isMemberVariableAssignment(tok, membervar))
+                        return true;
+
+                    if (isMemberVariableUsage(tok, var.isPointer(), membervar))
+                        uninitStructMemberError(tok, tok->str() + "." + membervar);
+                }
+                continue;
+            }
+
+            if (isVariableUsage(tok, var.isPointer(), _tokenizer->isCPP())) {
                 if (!suppressErrors)
                     uninitvarError(tok, tok->str());
                 else
@@ -1267,13 +1440,95 @@ bool CheckUninitVar::checkIfForWhileHead(const Token *startparanthesis, const Va
     return false;
 }
 
-bool CheckUninitVar::isVariableUsage(const Token *vartok, bool pointer) const
+bool CheckUninitVar::checkLoopBody(const Token *tok, const Variable& var, const std::string &membervar, const bool suppressErrors)
+{
+    const Token *usetok = NULL;
+
+    assert(tok->str() == "{");
+
+    for (const Token * const end = tok->link(); tok != end; tok = tok->next()) {
+        if (tok->varId() == var.varId()) {
+            if (!membervar.empty()) {
+                if (isMemberVariableAssignment(tok, membervar))
+                    return true;
+
+                if (isMemberVariableUsage(tok, var.isPointer(), membervar))
+                    usetok = tok;
+            } else {
+                if (isVariableUsage(tok, var.isPointer(), _tokenizer->isCPP()))
+                    usetok = tok;
+                else
+                    return true;
+            }
+        }
+
+        if (Token::Match(tok, "sizeof|typeof ("))
+            tok = tok->next()->link();
+
+        if (Token::Match(tok, "asm ( %str% ) ;"))
+            return true;
+    }
+
+    if (!suppressErrors && usetok) {
+        if (membervar.empty())
+            uninitvarError(usetok, usetok->str());
+        else
+            uninitStructMemberError(usetok, usetok->str() + "." + membervar);
+        return true;
+    }
+
+    return false;
+}
+
+bool CheckUninitVar::isVariableUsage(const Token *vartok, bool pointer, bool cpp)
 {
     if (vartok->previous()->str() == "return")
         return true;
 
-    if (Token::Match(vartok->previous(), "++|--|%op%")) {
-        if (vartok->previous()->str() == ">>" && _tokenizer->isCPP()) {
+    // Passing variable to function..
+    if (Token::Match(vartok->previous(), "[(,] %var% [,)]") || Token::Match(vartok->tokAt(-2), "[(,] & %var% [,)]")) {
+        const bool address(vartok->previous()->str() == "&");
+
+        // locate start parentheses in function call..
+        int argumentNumber = 0;
+        const Token *start = vartok;
+        while (start && !Token::Match(start, "[;{}(]")) {
+            if (start->str() == ")")
+                start = start->link();
+            else if (start->str() == ",")
+                ++argumentNumber;
+            start = start->previous();
+        }
+
+        // is this a function call?
+        if (start && Token::Match(start->previous(), "%var% (")) {
+            // check how function handle uninitialized data arguments..
+            const Function *func = start->previous()->function();
+            if (func) {
+                const Variable *arg = func->getArgumentVar(argumentNumber);
+                if (arg) {
+                    const Token *argStart = arg->typeStartToken();
+                    while (argStart->previous() && argStart->previous()->isName())
+                        argStart = argStart->previous();
+                    if (!address && Token::Match(argStart, "const| struct| %type% %var% [,)]"))
+                        return true;
+                    if (Token::Match(argStart, "const %type% & %var% [,)]"))
+                        return true;
+                    if (pointer && !address && Token::Match(argStart, "%type% * %var% [,)]"))
+                        return true;
+                    if ((pointer || address) && Token::Match(argStart, "const %type% * %var% [,)]"))
+                        return true;
+                }
+
+            } else if (Token::Match(start->previous(), "if|while|for")) {
+                // control-flow statement reading the variable "by value"
+                return true;
+            }
+        }
+    }
+
+    if (Token::Match(vartok->previous(), "++|--|%cop%")) {
+        if (cpp && vartok->previous()->str() == ">>") {
             // assume that variable is initialized
             return false;
         }
@@ -1281,8 +1536,12 @@ bool CheckUninitVar::isVariableUsage(const Token *vartok, bool pointer) const
         // is there something like: ; "*((&var ..expr.. ="  => the variable is assigned
         if (vartok->previous()->str() == "&") {
             const Token *tok2 = vartok->tokAt(-2);
-            if (Token::simpleMatch(tok2,")"))
+            if (tok2 && (tok2->isConstOp() || tok2->str() == "("))
+                return false; // address of
+            if (tok2 && tok2->str() == ")")
                 tok2 = tok2->link()->previous();
+            if (Token::Match(tok2,") ( %type% *| ) &") && tok2->tokAt(2)->varId() == 0)
+                return false; // cast
             while (tok2 && tok2->str() == "(")
                 tok2 = tok2->previous();
             while (tok2 && tok2->str() == "*")
@@ -1305,7 +1564,7 @@ bool CheckUninitVar::isVariableUsage(const Token *vartok, bool pointer) const
     }
 
     bool unknown = false;
-    if (pointer && CheckNullPointer::isPointerDeRef(vartok, unknown, _tokenizer->getSymbolDatabase())) {
+    if (pointer && CheckNullPointer::isPointerDeRef(vartok, unknown)) {
         // function parameter?
         bool functionParameter = false;
         if (Token::Match(vartok->tokAt(-2), "%var% (") || vartok->previous()->str() == ",")
@@ -1316,16 +1575,21 @@ bool CheckUninitVar::isVariableUsage(const Token *vartok, bool pointer) const
             return true;
     }
 
-    if (_tokenizer->isCPP() && Token::Match(vartok->next(), "<<|>>")) {
+    if (cpp && Token::Match(vartok->next(), "<<|>>")) {
+        // Is this calculation done in rhs?
+        const Token *tok = vartok;
+        while (tok && Token::Match(tok, "%var%|.|::"))
+            tok = tok->previous();
+        if (Token::Match(tok, "[;{}]"))
+            return false;
+
         // Is variable a known POD type then this is a variable usage,
         // otherwise we assume it's not.
-        const Variable *var = _tokenizer->getSymbolDatabase()->getVariableFromVarId(vartok->varId());
-        if (var && var->typeStartToken()->isStandardType())
-            return true;
-        return false;
+        const Variable *var = vartok->variable();
+        return (var && var->typeStartToken()->isStandardType());
     }
 
-    if (Token::Match(vartok->next(), "++|--|%op%"))
+    if (vartok->next() && vartok->next()->isOp() && !vartok->next()->isAssignmentOp())
         return true;
 
     if (vartok->strAt(1) == "]")
@@ -1334,6 +1598,36 @@ bool CheckUninitVar::isVariableUsage(const Token *vartok, bool pointer) const
     return false;
 }
 
+bool CheckUninitVar::isMemberVariableAssignment(const Token *tok, const std::string &membervar) const
+{
+    if (Token::Match(tok, "%var% . %var%") && tok->strAt(2) == membervar) {
+        if (Token::Match(tok->tokAt(3), "[=.[]"))
+            return true;
+        else if (Token::Match(tok->tokAt(-2), "[(,=] &"))
+            return true;
+        else if ((tok->previous() && tok->previous()->isConstOp()) || Token::Match(tok->previous(), "[|="))
+            ; // member variable usage
+        else
+            return true;
+    } else if (tok->strAt(1) == "=")
+        return true;
+    else if (tok->strAt(-1) == "&")
+        return true;
+    return false;
+}
+
+bool CheckUninitVar::isMemberVariableUsage(const Token *tok, bool isPointer, const std::string &membervar) const
+{
+    if (isMemberVariableAssignment(tok, membervar))
+        return false;
+
+    if (Token::Match(tok, "%var% . %var%") && tok->strAt(2) == membervar)
+        return true;
+    else if (Token::Match(tok->previous(), "[(,] %var% [,)]") && isVariableUsage(tok, isPointer, _tokenizer->isCPP()))
+        return true;
+
+    return false;
+}
 
 void CheckUninitVar::uninitstringError(const Token *tok, const std::string &varname, bool strncpy_)
 {
@@ -1349,3 +1643,11 @@ void CheckUninitVar::uninitvarError(const Token *tok, const std::string &varname
 {
     reportError(tok, Severity::error, "uninitvar", "Uninitialized variable: " + varname);
 }
+
+void CheckUninitVar::uninitStructMemberError(const Token *tok, const std::string &membername)
+{
+    reportError(tok,
+                Severity::error,
+                "uninitStructMember",
+                "Uninitialized struct member: " + membername);
+}
