@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,13 +47,6 @@ public:
         : Check(myName(), tokenizer, settings, errorLogger)
     { }
 
-    /** @brief Run checks against the normal token list */
-    void runChecks(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger) {
-        (void)tokenizer;
-        (void)settings;
-        (void)errorLogger;
-    }
-
     /** @brief Run checks against the simplified token list */
     void runSimplifiedChecks(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger) {
         CheckUninitVar checkUninitVar(tokenizer, settings, errorLogger);
@@ -64,10 +57,12 @@ public:
     /** Check for uninitialized variables */
     void check();
     void checkScope(const Scope* scope);
-    bool checkScopeForVariable(const Token *tok, const Variable& var, bool * const possibleInit);
-    bool checkIfForWhileHead(const Token *startparanthesis, const Variable& var, bool suppressErrors, bool isuninit);
-    bool isVariableUsage(const Token *vartok, bool ispointer) const;
-
+    bool checkScopeForVariable(const Scope* scope, const Token *tok, const Variable& var, bool * const possibleInit, bool * const noreturn, const std::string &membervar);
+    bool checkIfForWhileHead(const Token *startparentheses, const Variable& var, bool suppressErrors, bool isuninit, const std::string &membervar);
+    bool checkLoopBody(const Token *tok, const Variable& var, const std::string &membervar, const bool suppressErrors);
+    static bool isVariableUsage(const Token *vartok, bool ispointer, bool cpp);
+    bool isMemberVariableAssignment(const Token *tok, const std::string &membervar) const;
+    bool isMemberVariableUsage(const Token *tok, bool isPointer, const std::string &membervar) const;
 
     /**
      * @brief Uninitialized variables: analyse functions to see how they work with uninitialized variables
@@ -85,6 +80,7 @@ public:
     void uninitstringError(const Token *tok, const std::string &varname, bool strncpy_);
     void uninitdataError(const Token *tok, const std::string &varname);
     void uninitvarError(const Token *tok, const std::string &varname);
+    void uninitStructMemberError(const Token *tok, const std::string &membername);
 
 private:
     void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings) const {
@@ -94,6 +90,7 @@ private:
         c.uninitstringError(0, "varname", true);
         c.uninitdataError(0, "varname");
         c.uninitvarError(0, "varname");
+        c.uninitStructMemberError(0, "a.b");
     }
 
     static std::string myName() {
