@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +20,8 @@
 //---------------------------------------------------------------------------
 #include "checkunusedvar.h"
 #include "symboldatabase.h"
-
+#include <algorithm>
+#include <cctype>
 //---------------------------------------------------------------------------
 
 // Register this check class (by creating a static instance of it)
@@ -54,7 +55,8 @@ public:
         }
 
         /** variable is used.. set both read+write */
-        void use() {
+        void use(std::list<std::set<unsigned int> > & varReadInScope) {
+            varReadInScope.back().insert(_var->varId());
             _read = true;
             _write = true;
         }
@@ -76,12 +78,28 @@ public:
         bool _allocateMemory;
     };
 
-    typedef std::map<unsigned int, VariableUsage> VariableMap;
+    class ScopeGuard {
+    public:
+        ScopeGuard(Variables & guarded,
+                   bool insideLoop)
+            :_guarded(guarded),
+             _insideLoop(insideLoop) {
+            _guarded.enterScope();
+        }
+
+        ~ScopeGuard() {
+            _guarded.leaveScope(_insideLoop);
+        }
+
+    private:
+        Variables & _guarded;
+        bool _insideLoop;
+    };
 
     void clear() {
         _varUsage.clear();
     }
-    const VariableMap &varUsage() const {
+    const std::map<unsigned int, VariableUsage> &varUsage() const {
         return _varUsage;
     }
     void addVar(const Variable *var, VariableType type, bool write_);
@@ -103,8 +121,17 @@ public:
     void eraseAll(unsigned int varid);
     void clearAliases(unsigned int varid);
 
+    ScopeGuard newScope(bool insideLoop) {
+        return ScopeGuard(*this, insideLoop);
+    }
+
 private:
-    VariableMap _varUsage;
+    void enterScope();
+    void leaveScope(bool insideLoop);
+
+    std::map<unsigned int, VariableUsage> _varUsage;
+    std::list<std::set<unsigned int> > _varAddedInScope;
+    std::list<std::set<unsigned int> > _varReadInScope;
 };
 
 
@@ -123,7 +150,7 @@ void Variables::alias(unsigned int varid1, unsigned int varid2, bool replace)
     // alias to self
     if (varid1 == varid2) {
         if (var1)
-            var1->use();
+            var1->use(_varReadInScope);
         return;
     }
 
@@ -150,8 +177,10 @@ void Variables::alias(unsigned int varid1, unsigned int varid2, bool replace)
     var2->_aliases.insert(varid1);
     var1->_aliases.insert(varid2);
 
-    if (var2->_type == Variables::pointer)
+    if (var2->_type == Variables::pointer) {
+        _varReadInScope.back().insert(varid2);
         var2->_read = true;
+    }
 }
 
 void Variables::clearAliases(unsigned int varid)
@@ -196,8 +225,10 @@ void Variables::addVar(const Variable *var,
                        VariableType type,
                        bool write_)
 {
-    if (var->varId() > 0)
+    if (var->varId() > 0) {
+        _varAddedInScope.back().insert(var->varId());
         _varUsage.insert(std::make_pair(var->varId(), VariableUsage(var, type, false, write_, false)));
+    }
 }
 
 void Variables::allocateMemory(unsigned int varid, const Token* tok)
@@ -215,8 +246,10 @@ void Variables::read(unsigned int varid, const Token* tok)
     VariableUsage *usage = find(varid);
 
     if (usage) {
+        _varReadInScope.back().insert(varid);
         usage->_read = true;
-        usage->_lastAccess = tok;
+        if (tok)
+            usage->_lastAccess = tok;
     }
 }
 
@@ -231,6 +264,7 @@ void Variables::readAliases(unsigned int varid, const Token* tok)
             VariableUsage *aliased = find(*aliases);
 
             if (aliased) {
+                _varReadInScope.back().insert(*aliases);
                 aliased->_read = true;
                 aliased->_lastAccess = tok;
             }
@@ -285,7 +319,7 @@ void Variables::use(unsigned int varid, const Token* tok)
     VariableUsage *usage = find(varid);
 
     if (usage) {
-        usage->use();
+        usage->use(_varReadInScope);
         usage->_lastAccess = tok;
 
         std::set<unsigned int>::iterator aliases;
@@ -294,7 +328,7 @@ void Variables::use(unsigned int varid, const Token* tok)
             VariableUsage *aliased = find(*aliases);
 
             if (aliased) {
-                aliased->use();
+                aliased->use(_varReadInScope);
                 aliased->_lastAccess = tok;
             }
         }
@@ -325,13 +359,51 @@ void Variables::modified(unsigned int varid, const Token* tok)
 Variables::VariableUsage *Variables::find(unsigned int varid)
 {
     if (varid) {
-        VariableMap::iterator i = _varUsage.find(varid);
+        std::map<unsigned int, VariableUsage>::iterator i = _varUsage.find(varid);
         if (i != _varUsage.end())
             return &i->second;
     }
     return 0;
 }
 
+void Variables::enterScope()
+{
+    _varAddedInScope.push_back(std::set<unsigned int>());
+    _varReadInScope.push_back(std::set<unsigned int>());
+}
+
+void Variables::leaveScope(bool insideLoop)
+{
+    if (insideLoop) {
+        // read variables are read again in subsequent run through loop
+        std::set<unsigned int> const & currentVarReadInScope = _varReadInScope.back();
+        for (std::set<unsigned int>::const_iterator readIter = currentVarReadInScope.begin();
+             readIter != currentVarReadInScope.end();
+             ++readIter) {
+            read(*readIter, NULL);
+        }
+    }
+
+    std::list<std::set<unsigned int> >::reverse_iterator reverseReadIter = _varReadInScope.rbegin();
+    ++reverseReadIter;
+    if (reverseReadIter != _varReadInScope.rend()) {
+        // Transfer read variables into previous scope
+
+        std::set<unsigned int> const & currentVarAddedInScope = _varAddedInScope.back();
+        std::set<unsigned int>  & currentVarReadInScope = _varReadInScope.back();
+        for (std::set<unsigned int>::const_iterator addedIter = currentVarAddedInScope.begin();
+             addedIter != currentVarAddedInScope.end();
+             ++addedIter) {
+            currentVarReadInScope.erase(*addedIter);
+        }
+        std::set<unsigned int> & previousVarReadInScope = *reverseReadIter;
+        previousVarReadInScope.insert(currentVarReadInScope.begin(),
+                                      currentVarReadInScope.end());
+    }
+    _varReadInScope.pop_back();
+    _varAddedInScope.pop_back();
+}
+
 static const Token* doAssignment(Variables &variables, const Token *tok, bool dereference, const Scope *scope)
 {
     // a = a + b;
@@ -355,9 +427,13 @@ static const Token* doAssignment(Variables &variables, const Token *tok, bool de
         }
         tok = tok->next();
 
-        if (Token::Match(tok, "&| %var%") ||
-            Token::Match(tok, "( const| struct|union| %type% *| ) &| %var%") ||
-            Token::Match(tok, "( const| struct|union| %type% *| ) ( &| %var%") ||
+        if (Token::Match(tok, "( const| struct|union| %type% * ) ( ("))
+            tok = tok->link()->next();
+
+        if (Token::Match(tok, "( [(<] const| struct|union| %type% *| [>)]"))
+            tok = tok->next();
+
+        if (Token::Match(tok, "(| &| %var%") ||
             Token::Match(tok->next(), "< const| struct|union| %type% *| > ( &| %var%")) {
             bool addressOf = false;
 
@@ -373,11 +449,9 @@ static const Token* doAssignment(Variables &variables, const Token *tok, bool de
                 if (Token::Match(tok, "struct|union"))
                     tok = tok->next();
 
-                tok = tok->next();
-                if (tok->str() == "*")
+                while ((tok->isName() && tok->varId() == 0) || (tok->str() == "*") || (tok->str() == ")"))
                     tok = tok->next();
 
-                tok = tok->next();
                 if (tok->str() == "&") {
                     addressOf = true;
                     tok = tok->next();
@@ -475,13 +549,13 @@ static const Token* doAssignment(Variables &variables, const Token *tok, bool de
                             else
                                 variables.read(varid2, tok);
                         } else {
-                            variables.read(varid2, tok);
+                            variables.readAll(varid2, tok);
                         }
                     }
                 } else if (var1->_type == Variables::reference) {
                     variables.alias(varid1, varid2, true);
                 } else {
-                    if (var2->_type == Variables::pointer && tok->strAt(1) == "[")
+                    if ((var2->_type == Variables::pointer || var2->_type == Variables::pointerArray) && tok->strAt(1) == "[")
                         variables.readAliases(varid2, tok);
 
                     variables.read(varid2, tok);
@@ -517,7 +591,7 @@ static const Token* doAssignment(Variables &variables, const Token *tok, bool de
     // check for alias to struct member
     // char c[10]; a.b = c;
     else if (Token::Match(tok->tokAt(-2), "%var% .")) {
-        if (Token::Match(tok->tokAt(2), "%var%")) {
+        if (tok->tokAt(2)->varId()) {
             unsigned int varid2 = tok->tokAt(2)->varId();
             Variables::VariableUsage *var2 = variables.find(varid2);
 
@@ -531,18 +605,28 @@ static const Token* doAssignment(Variables &variables, const Token *tok, bool de
         }
     }
 
+    // Possible pointer alias
+    else if (Token::Match(tok, "%var% = %var% ;")) {
+        const unsigned int varid2 = tok->tokAt(2)->varId();
+        Variables::VariableUsage *var2 = variables.find(varid2);
+        if (var2 && (var2->_type == Variables::array ||
+                     var2->_type == Variables::pointer)) {
+            variables.use(varid2,tok);
+        }
+    }
+
     return tok;
 }
 
-static bool isRecordTypeWithoutSideEffects(const Scope* type)
+static bool isRecordTypeWithoutSideEffects(const Type* type)
 {
     // a type that has no side effects (no constructors and no members with constructors)
     /** @todo false negative: check constructors for side effects */
-    if (type && type->numConstructors == 0 &&
-        (type->varlist.empty() || type->needInitialization == Scope::True)) {
+    if (type && type->classScope && type->classScope->numConstructors == 0 &&
+        (type->classScope->varlist.empty() || type->needInitialization == Type::True)) {
         bool yes = true;
-        for (std::vector<Scope::BaseInfo>::const_iterator i = type->derivedFrom.begin(); yes && i != type->derivedFrom.end(); ++i)
-            yes = isRecordTypeWithoutSideEffects(i->scope);
+        for (std::vector<Type::BaseInfo>::const_iterator i = type->derivedFrom.begin(); yes && i != type->derivedFrom.end(); ++i)
+            yes = isRecordTypeWithoutSideEffects(i->type);
         return yes;
     }
     return false;
@@ -589,8 +673,10 @@ static const Token * skipBracketsAndMembers(const Token *tok)
 //---------------------------------------------------------------------------
 // Usage of function variables
 //---------------------------------------------------------------------------
-void CheckUnusedVar::checkFunctionVariableUsage_iterateScopes(const Scope* const scope, Variables& variables, bool insideLoop, std::vector<unsigned int> &usedVariables)
+void CheckUnusedVar::checkFunctionVariableUsage_iterateScopes(const Scope* const scope, Variables& variables, bool insideLoop)
 {
+    Variables::ScopeGuard scopeGuard=variables.newScope(insideLoop);
+
     // Find declarations if the scope is executable..
     if (scope->isExecutable()) {
         // Find declarations
@@ -610,7 +696,12 @@ void CheckUnusedVar::checkFunctionVariableUsage_iterateScopes(const Scope* const
                 type = Variables::pointerPointer;
             else if (i->isPointer())
                 type = Variables::pointer;
-            else if (_tokenizer->isC() || i->typeEndToken()->isStandardType() || isRecordTypeWithoutSideEffects(i->type()) || Token::simpleMatch(i->typeStartToken(), "std ::"))
+            else if (_tokenizer->isC() ||
+                     i->typeEndToken()->isStandardType() ||
+                     isRecordTypeWithoutSideEffects(i->type()) ||
+                     (Token::simpleMatch(i->typeStartToken(), "std ::") &&
+                      i->typeStartToken()->strAt(2) != "lock_guard" &&
+                      i->typeStartToken()->strAt(2) != "unique_lock"))
                 type = Variables::standard;
             if (type == Variables::none || isPartOfClassStructUnion(i->typeStartToken()))
                 continue;
@@ -634,7 +725,7 @@ void CheckUnusedVar::checkFunctionVariableUsage_iterateScopes(const Scope* const
             if (defValTok && defValTok->str() == "=") {
                 if (defValTok->next() && defValTok->next()->str() == "{") {
                     for (const Token* tok = defValTok; tok && tok != defValTok->linkAt(1); tok = tok->next())
-                        if (Token::Match(tok, "%var%")) // Variables used to initialize the array read.
+                        if (tok->varId()) // Variables used to initialize the array read.
                             variables.read(tok->varId(), i->nameToken());
                 } else
                     doAssignment(variables, i->nameToken(), false, scope);
@@ -643,21 +734,12 @@ void CheckUnusedVar::checkFunctionVariableUsage_iterateScopes(const Scope* const
         }
     }
 
-    if (scope->classDef->str() == "for" || scope->classDef->str() == "while") {
-        insideLoop = true;
-    }
-
     // Check variable usage
     for (const Token *tok = scope->classDef->next(); tok && tok != scope->classEnd; tok = tok->next()) {
-        if (tok->str() == "for" || tok->str() == "while") {
+        if (tok->str() == "for" || tok->str() == "while" || tok->str() == "do") {
             for (std::list<Scope*>::const_iterator i = scope->nestedList.begin(); i != scope->nestedList.end(); ++i) {
                 if ((*i)->classDef == tok) { // Find associated scope
-                    checkFunctionVariableUsage_iterateScopes(*i, variables, true, usedVariables); // Scan child scope
-                    insideLoop = false;
-                    std::vector<unsigned int>::iterator it;
-                    for (it = usedVariables.begin(); it != usedVariables.end(); ++it) {
-                        variables.read((*it), tok);
-                    }
+                    checkFunctionVariableUsage_iterateScopes(*i, variables, true); // Scan child scope
                     tok = (*i)->classStart->link();
                     break;
                 }
@@ -668,7 +750,7 @@ void CheckUnusedVar::checkFunctionVariableUsage_iterateScopes(const Scope* const
         if (tok->str() == "{") {
             for (std::list<Scope*>::const_iterator i = scope->nestedList.begin(); i != scope->nestedList.end(); ++i) {
                 if ((*i)->classStart == tok) { // Find associated scope
-                    checkFunctionVariableUsage_iterateScopes(*i, variables, insideLoop, usedVariables); // Scan child scope
+                    checkFunctionVariableUsage_iterateScopes(*i, variables, false); // Scan child scope
                     tok = tok->link();
                     break;
                 }
@@ -681,18 +763,47 @@ void CheckUnusedVar::checkFunctionVariableUsage_iterateScopes(const Scope* const
             variables.clear();
             break;
         }
+        if (Token::Match(tok, "goto")) { // https://sourceforge.net/apps/trac/cppcheck/ticket/4447
+            variables.clear();
+            break;
+        }
+
+
+        // bailout when for_each is used
+        if (Token::Match(tok,"%var% (") && Token::simpleMatch(tok->linkAt(1),") {")) {
+            // does the name contain "for_each" or "foreach"?
+            std::string nameTok;
+            nameTok.resize(tok->str().size());
+            std::transform(tok->str().begin(), tok->str().end(), nameTok.begin(), ::tolower);
+            if (nameTok.find("foreach") != std::string::npos || nameTok.find("for_each") != std::string::npos) {
+                // bailout all variables in the body that are used more than once.
+                // TODO: there is no need to bailout if variable is only read or only written
+                std::set<unsigned int> varid;
+                const Token * const endTok = tok->linkAt(1)->linkAt(1);
+                for (const Token *tok2 = endTok->link(); tok2 && tok2 != endTok; tok2 = tok2->next()) {
+                    if (tok2->varId()) {
+                        if (varid.find(tok2->varId()) == varid.end())
+                            varid.insert(tok2->varId());
+                        else
+                            variables.erase(tok2->varId());
+                    }
+                }
+            }
+        }
 
         if (Token::Match(tok->previous(), "[;{}]")) {
             for (const Token* tok2 = tok->next(); tok2; tok2 = tok2->next()) {
                 if (tok2->varId()) {
-                    const Variable* var = _tokenizer->getSymbolDatabase()->getVariableFromVarId(tok2->varId());
+                    const Variable *var = tok2->variable();
                     if (var && var->nameToken() == tok2) { // Declaration: Skip
                         tok = tok2->next();
                         if (Token::Match(tok, "( %var% )")) // Simple initialization through copy ctor
                             tok = tok->next();
-                        else if (Token::Match(tok, "= %var% ;")) // Simple initialization
+                        else if (Token::Match(tok, "= %var% ;")) { // Simple initialization
                             tok = tok->next();
-                        else if (var->typeEndToken()->str() == ">") // Be careful with types like std::vector
+                            if (!var->isReference())
+                                variables.read(tok->varId(), tok);
+                        } else if (var->typeEndToken()->str() == ">") // Be careful with types like std::vector
                             tok = tok->previous();
                         break;
                     }
@@ -735,9 +846,8 @@ void CheckUnusedVar::checkFunctionVariableUsage_iterateScopes(const Scope* const
             variables.use(tok->tokAt(2)->varId(), tok);
         }
         // assignment
-        else if (!Token::Match(tok->tokAt(-2), "[;{}.] %var% (") &&
-                 (Token::Match(tok, "*| ++|--| %var% ++|--| =") ||
-                  Token::Match(tok, "*| ( const| %type% *| ) %var% ="))) {
+        else if (Token::Match(tok, "*| ++|--| %var% ++|--| =") ||
+                 Token::Match(tok, "*| ( const| %type% *| ) %var% =")) {
             bool dereference = false;
             bool pre = false;
             bool post = false;
@@ -762,7 +872,7 @@ void CheckUnusedVar::checkFunctionVariableUsage_iterateScopes(const Scope* const
                 post = true;
 
             const unsigned int varid1 = tok->varId();
-            const Token *start = tok;
+            const Token * const start = tok;
 
             tok = doAssignment(variables, tok, dereference, scope);
 
@@ -777,7 +887,9 @@ void CheckUnusedVar::checkFunctionVariableUsage_iterateScopes(const Scope* const
                 variables.read(varid1, tok);
             } else {
                 Variables::VariableUsage *var = variables.find(varid1);
-                if (var && var->_type == Variables::reference) {
+                if (var && start->strAt(-1) == ",") {
+                    variables.use(varid1, tok);
+                } else if (var && var->_type == Variables::reference) {
                     variables.writeAliases(varid1, tok);
                     variables.read(varid1, tok);
                 }
@@ -796,7 +908,7 @@ void CheckUnusedVar::checkFunctionVariableUsage_iterateScopes(const Scope* const
 
                         // is it a user defined type?
                         if (!type->isStandardType()) {
-                            const Variable* variable = _tokenizer->getSymbolDatabase()->getVariableFromVarId(start->varId());
+                            const Variable *variable = start->variable();
                             if (!variable || !isRecordTypeWithoutSideEffects(variable->type()))
                                 allocate = false;
                         }
@@ -830,10 +942,12 @@ void CheckUnusedVar::checkFunctionVariableUsage_iterateScopes(const Scope* const
 
             // checked for chained assignments
             if (tok != start && equal && equal->str() == "=") {
-                Variables::VariableUsage *var = variables.find(tok->varId());
+                unsigned int varId = tok->varId();
+                Variables::VariableUsage *var = variables.find(varId);
 
-                if (var && var->_type != Variables::reference)
-                    var->_read = true;
+                if (var && var->_type != Variables::reference) {
+                    variables.read(varId,tok);
+                }
 
                 tok = tok->previous();
             }
@@ -866,67 +980,58 @@ void CheckUnusedVar::checkFunctionVariableUsage_iterateScopes(const Scope* const
             }
         }
 
+        else if (_tokenizer->isCPP() && Token::Match(tok, "[;{}] %var% <<")) {
+            variables.erase(tok->next()->varId());
+        }
+
         else if (Token::Match(tok, "& %var%")) {
             if (tok->previous()->isName() || tok->previous()->isNumber()) { // bitop
                 variables.read(tok->next()->varId(), tok);
             } else // addressof
                 variables.use(tok->next()->varId(), tok); // use = read + write
-            usedVariables.push_back(tok->next()->varId());
         } else if (Token::Match(tok, ">> %var%")) {
             variables.use(tok->next()->varId(), tok); // use = read + write
-            usedVariables.push_back(tok->next()->varId());
         } else if (Token::Match(tok, "%var% >>|&") && Token::Match(tok->previous(), "[{};:]")) {
             variables.read(tok->varId(), tok);
-            usedVariables.push_back(tok->next()->varId());
         }
 
         // function parameter
         else if (Token::Match(tok, "[(,] %var% [")) {
             variables.use(tok->next()->varId(), tok);   // use = read + write
-            usedVariables.push_back(tok->next()->varId());
         } else if (Token::Match(tok, "[(,] %var% [,)]") && tok->previous()->str() != "*") {
             variables.use(tok->next()->varId(), tok);   // use = read + write
-            usedVariables.push_back(tok->next()->varId());
         } else if (Token::Match(tok, "[(,] (") &&
                    Token::Match(tok->next()->link(), ") %var% [,)]")) {
             variables.use(tok->next()->link()->next()->varId(), tok);   // use = read + write
-            usedVariables.push_back(tok->next()->link()->next()->varId());
         }
 
         // function
         else if (Token::Match(tok, "%var% (")) {
             variables.read(tok->varId(), tok);
-            usedVariables.push_back(tok->varId());
             if (Token::Match(tok->tokAt(2), "%var% =")) {
                 variables.read(tok->tokAt(2)->varId(), tok);
-                usedVariables.push_back(tok->tokAt(2)->varId());
             }
         }
 
         else if (Token::Match(tok, "[{,] %var% [,}]")) {
             variables.read(tok->next()->varId(), tok);
-            usedVariables.push_back(tok->next()->varId());
         }
 
         else if (Token::Match(tok, "%var% .")) {
             variables.use(tok->varId(), tok);   // use = read + write
-            usedVariables.push_back(tok->varId());
         }
 
-        else if (tok->isExtendedOp() &&
-                 Token::Match(tok->next(), "%var%") && !Token::Match(tok->next(), "true|false|new") && tok->strAt(2) != "=") {
+        else if (tok->isExtendedOp() && tok->next() &&
+                 tok->next()->varId() && !Token::Match(tok->next(), "true|false|new") && tok->strAt(2) != "=") {
             variables.readAll(tok->next()->varId(), tok);
-            usedVariables.push_back(tok->next()->varId());
         }
 
-        else if (Token::Match(tok, "%var%") && tok->next() && (tok->next()->str() == ")" || tok->next()->isExtendedOp())) {
+        else if (tok->varId() && tok->next() && (tok->next()->str() == ")" || tok->next()->isExtendedOp())) {
             variables.readAll(tok->varId(), tok);
-            usedVariables.push_back(tok->varId());
         }
 
         else if (Token::Match(tok, "%var% ;") && Token::Match(tok->previous(), "[;{}:]")) {
             variables.readAll(tok->varId(), tok);
-            usedVariables.push_back(tok->varId());
         }
 
         else if (Token::Match(tok, "++|-- %var%")) {
@@ -934,7 +1039,6 @@ void CheckUnusedVar::checkFunctionVariableUsage_iterateScopes(const Scope* const
                 variables.use(tok->next()->varId(), tok);
             else
                 variables.modified(tok->next()->varId(), tok);
-            usedVariables.push_back(tok->next()->varId());
         }
 
         else if (Token::Match(tok, "%var% ++|--")) {
@@ -942,7 +1046,6 @@ void CheckUnusedVar::checkFunctionVariableUsage_iterateScopes(const Scope* const
                 variables.use(tok->varId(), tok);
             else
                 variables.modified(tok->varId(), tok);
-            usedVariables.push_back(tok->varId());
         }
 
         else if (tok->isAssignmentOp()) {
@@ -952,7 +1055,6 @@ void CheckUnusedVar::checkFunctionVariableUsage_iterateScopes(const Scope* const
                         variables.write(tok2->varId(), tok);
                     else
                         variables.read(tok2->varId(), tok);
-                    usedVariables.push_back(tok2->varId());
                 }
             }
         }
@@ -967,20 +1069,21 @@ void CheckUnusedVar::checkFunctionVariableUsage()
     // Parse all executing scopes..
     const SymbolDatabase *symbolDatabase = _tokenizer->getSymbolDatabase();
 
-    for (std::list<Scope>::const_iterator scope = symbolDatabase->scopeList.begin(); scope != symbolDatabase->scopeList.end(); ++scope) {
-        // only check functions
-        if (scope->type != Scope::eFunction)
-            continue;
+    // only check functions
+    const std::size_t functions = symbolDatabase->functionScopes.size();
+    for (std::size_t i = 0; i < functions; ++i) {
+        const Scope * scope = symbolDatabase->functionScopes[i];
 
         // varId, usage {read, write, modified}
         Variables variables;
 
-        std::vector<unsigned int> usedVariables;
-        checkFunctionVariableUsage_iterateScopes(&*scope, variables, false, usedVariables);
+        checkFunctionVariableUsage_iterateScopes(&*scope, variables, false);
 
 
         // Check usage of all variables in the current scope..
-        for (Variables::VariableMap::const_iterator it = variables.varUsage().begin(); it != variables.varUsage().end(); ++it) {
+        for (std::map<unsigned int, Variables::VariableUsage>::const_iterator it = variables.varUsage().begin();
+             it != variables.varUsage().end();
+             ++it) {
             const Variables::VariableUsage &usage = it->second;
             const std::string &varname = usage._var->name();
             const Variable* var = symbolDatabase->getVariableFromVarId(it->first);
@@ -1053,12 +1156,6 @@ void CheckUnusedVar::checkStructMemberUsage()
             continue;
 
         if (Token::Match(tok, "struct|union %type% {")) {
-            structname.clear();
-            if (tok->strAt(-1) == "extern")
-                continue;
-            if ((!tok->previous() || tok->previous()->str() == ";") && Token::Match(tok->linkAt(2), ("} ; " + tok->strAt(1) + " %var% ;").c_str()))
-                continue;
-
             structname = tok->strAt(1);
 
             // Bail out if struct/union contain any functions
@@ -1081,6 +1178,32 @@ void CheckUnusedVar::checkStructMemberUsage()
             if (Token::findmatch(tok, s.c_str()))
                 structname.clear();
 
+            // Bail out if instance is initialized with {}..
+            if (!structname.empty()) {
+                const std::string pattern1(structname + " %var% ;");
+                const Token *tok2 = tok;
+                while (NULL != (tok2 = Token::findmatch(tok2->next(), pattern1.c_str()))) {
+                    if (Token::simpleMatch(tok2->tokAt(3), (tok2->strAt(1) + " = {").c_str())) {
+                        structname.clear();
+                        break;
+                    }
+                }
+            }
+
+            // bail out for extern/global struct
+            for (const Token *tok2 = Token::findmatch(tok, (structname + " %var%").c_str());
+                 tok2 && tok2->next();
+                 tok2 = Token::findmatch(tok2->next(), (structname + " %var%").c_str())) {
+
+                const Variable *var = tok2->next()->variable();
+                if (var && (var->isExtern() || (var->isGlobal() && !var->isStatic()))) {
+                    structname.clear();
+                    break;
+                }
+            }
+            if (structname.empty())
+                continue;
+
             // Try to prevent false positives when struct members are not used directly.
             if (Token::findmatch(tok, (structname + " *").c_str()))
                 structname.clear();
