@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 
 #include <algorithm>
 #include <fstream>
+#include <sstream>
 #include <stdexcept>
 #include "timer.h"
 
@@ -33,15 +34,14 @@
 #include <pcre.h>
 #endif
 
-static const char Version[] = "1.53";
+static const char Version[] = "1.59";
 static const char ExtraVersion[] = "";
 
 static TimerResults S_timerResults;
 
 CppCheck::CppCheck(ErrorLogger &errorLogger, bool useGlobalSuppressions)
-    : _useGlobalSuppressions(useGlobalSuppressions), _errorLogger(errorLogger)
+    : _errorLogger(errorLogger), exitcode(0), _useGlobalSuppressions(useGlobalSuppressions), tooManyConfigs(false)
 {
-    exitcode = 0;
 }
 
 CppCheck::~CppCheck()
@@ -50,11 +50,6 @@ CppCheck::~CppCheck()
         S_timerResults.ShowResults();
 }
 
-void CppCheck::settings(const Settings &currentSettings)
-{
-    _settings = currentSettings;
-}
-
 const char * CppCheck::version()
 {
     return Version;
@@ -67,28 +62,24 @@ const char * CppCheck::extraVersion()
 
 unsigned int CppCheck::check(const std::string &path)
 {
-    _filename = path;
-    return processFile();
+    return processFile(path);
 }
 
 unsigned int CppCheck::check(const std::string &path, const std::string &content)
 {
-    _filename = path;
     _fileContent = content;
-    const unsigned int retval = processFile();
+    const unsigned int retval = processFile(path);
     _fileContent.clear();
     return retval;
 }
 
-std::string CppCheck::replaceAll(std::string code, const std::string &from, const std::string &to)
+void CppCheck::replaceAll(std::string& code, const std::string &from, const std::string &to)
 {
-    size_t pos = 0;
+    std::size_t pos = 0;
     while ((pos = code.find(from, pos)) != std::string::npos) {
         code.replace(pos, from.length(), to);
         pos += to.length();
     }
-
-    return code;
 }
 
 bool CppCheck::findError(std::string code, const char FileName[])
@@ -105,7 +96,7 @@ bool CppCheck::findError(std::string code, const char FileName[])
     for (;;) {
 
         // Try to remove included files from the source
-        size_t found=previousCode.rfind("\n#endfile");
+        std::size_t found = previousCode.rfind("\n#endfile");
         if (found == std::string::npos) {
             // No modifications can be done to the code
         } else {
@@ -126,8 +117,8 @@ bool CppCheck::findError(std::string code, const char FileName[])
 
         // Add '\n' so that "\n#file" on first line would be found
         code = "// " + error + "\n" + code;
-        code = replaceAll(code, "\n#file", "\n// #file");
-        code = replaceAll(code, "\n#endfile", "\n// #endfile");
+        replaceAll(code, "\n#file", "\n// #file");
+        replaceAll(code, "\n#endfile", "\n// #endfile");
 
         // We have reduced the code as much as we can. Print out
         // the code and quit.
@@ -138,33 +129,19 @@ bool CppCheck::findError(std::string code, const char FileName[])
     return true;
 }
 
-unsigned int CppCheck::processFile()
+unsigned int CppCheck::processFile(const std::string& filename)
 {
     exitcode = 0;
 
-    // only show debug warnings for C/C++ source files (don't fix
-    // debug warnings for java/c#/etc files)
-    if (!Path::acceptFile(_filename))
+    // only show debug warnings for accepted C/C++ source files
+    if (!Path::acceptFile(filename))
         _settings.debugwarnings = false;
 
-    // TODO: Should this be moved out to its own function so all the files can be
-    // analysed before any files are checked?
-    if (_settings.test_2_pass && _settings._jobs == 1) {
-        const std::string printname = Path::toNativeSeparators(_filename);
-        reportOut("Analysing " + printname + "...");
-
-        std::ifstream f(_filename.c_str());
-        analyseFile(f, _filename);
-    }
-
-    _errout.str("");
-
     if (_settings.terminated())
         return exitcode;
 
     if (_settings._errorsOnly == false) {
-        std::string fixedpath(_filename);
-        fixedpath = Path::simplifyPath(fixedpath.c_str());
+        std::string fixedpath = Path::simplifyPath(filename.c_str());
         fixedpath = Path::toNativeSeparators(fixedpath);
         _errorLogger.reportOut(std::string("Checking ") + fixedpath + std::string("..."));
     }
@@ -177,86 +154,69 @@ unsigned int CppCheck::processFile()
         if (!_fileContent.empty()) {
             // File content was given as a string
             std::istringstream iss(_fileContent);
-            preprocessor.preprocess(iss, filedata, configurations, _filename, _settings._includePaths);
+            preprocessor.preprocess(iss, filedata, configurations, filename, _settings._includePaths);
         } else {
             // Only file name was given, read the content from file
-            std::ifstream fin(_filename.c_str());
+            std::ifstream fin(filename.c_str());
             Timer t("Preprocessor::preprocess", _settings._showtime, &S_timerResults);
-            preprocessor.preprocess(fin, filedata, configurations, _filename, _settings._includePaths);
+            preprocessor.preprocess(fin, filedata, configurations, filename, _settings._includePaths);
         }
 
         if (_settings.checkConfiguration) {
             return 0;
         }
 
-        _settings.ifcfg = bool(configurations.size() > 1);
-
         if (!_settings.userDefines.empty()) {
             configurations.clear();
             configurations.push_back(_settings.userDefines);
         }
 
-        int checkCount = 0;
+        if (!_settings._force && configurations.size() > _settings._maxConfigs) {
+            if (_settings.isEnabled("information")) {
+                tooManyConfigsError(Path::toNativeSeparators(filename),configurations.size());
+            } else {
+                tooManyConfigs = true;
+            }
+        }
+
+        unsigned int checkCount = 0;
         for (std::list<std::string>::const_iterator it = configurations.begin(); it != configurations.end(); ++it) {
             // Check only a few configurations (default 12), after that bail out, unless --force
             // was used.
-            if (!_settings._force && checkCount >= _settings._maxConfigs) {
-
-                const std::string fixedpath = Path::toNativeSeparators(_filename);
-                ErrorLogger::ErrorMessage::FileLocation location;
-                location.setfile(fixedpath);
-                std::list<ErrorLogger::ErrorMessage::FileLocation> loclist;
-                loclist.push_back(location);
-                const std::string msg("Interrupted checking because of too many #ifdef configurations.\n"
-                                      "The checking of the file was interrupted because there were too many "
-                                      "#ifdef configurations. Checking of all #ifdef configurations can be forced "
-                                      "by --force command line option or from GUI preferences. However that may "
-                                      "increase the checking time.");
-                ErrorLogger::ErrorMessage errmsg(loclist,
-                                                 Severity::information,
-                                                 msg,
-                                                 "toomanyconfigs",
-                                                 false);
-
-                if (!_settings.nomsg.isSuppressedLocal(errmsg._id, fixedpath, location.line)) {
-                    reportErr(errmsg);
-                }
-
+            if (!_settings._force && ++checkCount > _settings._maxConfigs)
                 break;
-            }
 
             cfg = *it;
-            Timer t("Preprocessor::getcode", _settings._showtime, &S_timerResults);
-            const std::string codeWithoutCfg = preprocessor.getcode(filedata, *it, _filename);
-            t.Stop();
 
             // If only errors are printed, print filename after the check
             if (_settings._errorsOnly == false && it != configurations.begin()) {
-                std::string fixedpath = Path::simplifyPath(_filename.c_str());
+                std::string fixedpath = Path::simplifyPath(filename.c_str());
                 fixedpath = Path::toNativeSeparators(fixedpath);
                 _errorLogger.reportOut(std::string("Checking ") + fixedpath + ": " + cfg + std::string("..."));
             }
 
+            Timer t("Preprocessor::getcode", _settings._showtime, &S_timerResults);
+            const std::string codeWithoutCfg = preprocessor.getcode(filedata, *it, filename, _settings.userDefines.empty());
+            t.Stop();
+
             const std::string &appendCode = _settings.append();
 
             if (_settings.debugFalsePositive) {
-                if (findError(codeWithoutCfg + appendCode, _filename.c_str())) {
+                if (findError(codeWithoutCfg + appendCode, filename.c_str())) {
                     return exitcode;
                 }
             } else {
-                checkFile(codeWithoutCfg + appendCode, _filename.c_str());
+                checkFile(codeWithoutCfg + appendCode, filename.c_str());
             }
-
-            ++checkCount;
         }
-    } catch (std::runtime_error &e) {
+    } catch (const std::runtime_error &e) {
         // Exception was thrown when checking this file..
-        const std::string fixedpath = Path::toNativeSeparators(_filename);
+        const std::string fixedpath = Path::toNativeSeparators(filename);
         _errorLogger.reportOut("Bailing out from checking " + fixedpath + ": " + e.what());
     }
 
     if (!_settings._errorsOnly)
-        reportUnmatchedSuppressions(_settings.nomsg.getUnmatchedLocalSuppressions(_filename));
+        reportUnmatchedSuppressions(_settings.nomsg.getUnmatchedLocalSuppressions(filename));
 
     _errorList.clear();
     return exitcode;
@@ -271,7 +231,6 @@ void CppCheck::checkFunctionUsage()
         const bool verbose_orig = _settings._verbose;
         _settings._verbose = false;
 
-        _errout.str("");
         if (_settings._errorsOnly == false)
             _errorLogger.reportOut("Checking usage of global functions..");
 
@@ -324,6 +283,8 @@ void CppCheck::checkFile(const std::string &code, const char FileName[])
         return;
 
     Tokenizer _tokenizer(&_settings, this);
+    if (_settings._showtime != SHOWTIME_NONE)
+        _tokenizer.setTimerResults(&S_timerResults);
     try {
         bool result;
 
@@ -338,6 +299,10 @@ void CppCheck::checkFile(const std::string &code, const char FileName[])
             return;
         }
 
+        // Update the _dependencies..
+        if (_tokenizer.list.getFiles().size() >= 2)
+            _dependencies.insert(_tokenizer.list.getFiles().begin()+1, _tokenizer.list.getFiles().end());
+
         // call all "runChecks" in all registered Check classes
         for (std::list<Check *>::iterator it = Check::instances().begin(); it != Check::instances().end(); ++it) {
             if (_settings.terminated())
@@ -403,14 +368,14 @@ void CppCheck::checkFile(const std::string &code, const char FileName[])
 
                     // determine location..
                     ErrorLogger::ErrorMessage::FileLocation loc;
-                    loc.setfile(_tokenizer.getFiles()->front());
+                    loc.setfile(_tokenizer.getSourceFilePath());
                     loc.line = 0;
 
                     unsigned int len = 0;
                     for (const Token *tok = _tokenizer.tokens(); tok; tok = tok->next()) {
                         len = len + 1 + tok->str().size();
                         if (len > pos1) {
-                            loc.setfile(_tokenizer.getFiles()->at(tok->fileIndex()));
+                            loc.setfile(_tokenizer.list.getFiles().at(tok->fileIndex()));
                             loc.line = tok->linenr();
                             break;
                         }
@@ -442,7 +407,7 @@ void CppCheck::checkFile(const std::string &code, const char FileName[])
         ErrorLogger::ErrorMessage::FileLocation loc;
         if (e.token) {
             loc.line = e.token->linenr();
-            const std::string fixedpath = Path::toNativeSeparators(_tokenizer.file(e.token));
+            const std::string fixedpath = Path::toNativeSeparators(_tokenizer.list.file(e.token));
             loc.setfile(fixedpath);
         } else {
             loc.setfile(_tokenizer.getSourceFilePath());
@@ -463,24 +428,64 @@ Settings &CppCheck::settings()
     return _settings;
 }
 
+void CppCheck::tooManyConfigsError(const std::string &file, const std::size_t numberOfConfigurations)
+{
+    if (!_settings.isEnabled("information") && !tooManyConfigs)
+        return;
+
+    tooManyConfigs = false;
+
+    if (_settings.isEnabled("information") && file.empty())
+        return;
+
+    std::list<ErrorLogger::ErrorMessage::FileLocation> loclist;
+    if (!file.empty()) {
+        ErrorLogger::ErrorMessage::FileLocation location;
+        location.setfile(file);
+        loclist.push_back(location);
+    }
+
+    std::ostringstream msg;
+    msg << "Too many #ifdef configurations - cppcheck only checks " << _settings._maxConfigs;
+    if (numberOfConfigurations > _settings._maxConfigs)
+        msg << " of " << numberOfConfigurations << " configurations. Use --force to check all configurations.\n";
+    if (file.empty())
+        msg << " configurations. Use --force to check all configurations. For more details, use --enable=information.\n";
+    msg << "The checking of the file will be interrupted because there are too many "
+        "#ifdef configurations. Checking of all #ifdef configurations can be forced "
+        "by --force command line option or from GUI preferences. However that may "
+        "increase the checking time.";
+    if (file.empty())
+        msg << " For more details, use --enable=information.";
+
+
+    ErrorLogger::ErrorMessage errmsg(loclist,
+                                     Severity::information,
+                                     msg.str(),
+                                     "toomanyconfigs",
+                                     false);
+
+    reportErr(errmsg);
+}
+
 //---------------------------------------------------------------------------
 
 void CppCheck::reportErr(const ErrorLogger::ErrorMessage &msg)
 {
-    const std::string errmsg = msg.toString(_settings._verbose);
+    std::string errmsg = msg.toString(_settings._verbose);
     if (errmsg.empty())
         return;
 
+    // Alert only about unique errors
+    if (std::find(_errorList.begin(), _errorList.end(), errmsg) != _errorList.end())
+        return;
+
     if (_settings.debugFalsePositive) {
         // Don't print out error
         _errorList.push_back(errmsg);
         return;
     }
 
-    // Alert only about unique errors
-    if (std::find(_errorList.begin(), _errorList.end(), errmsg) != _errorList.end())
-        return;
-
     std::string file;
     unsigned int line(0);
     if (!msg._callStack.empty()) {
@@ -500,14 +505,8 @@ void CppCheck::reportErr(const ErrorLogger::ErrorMessage &msg)
         exitcode = 1;
 
     _errorList.push_back(errmsg);
-    std::string errmsg2(errmsg);
-    if (_settings._verbose) {
-        errmsg2 += "\n    Defines=\'" + cfg + "\'\n";
-    }
 
     _errorLogger.reportErr(msg);
-
-    _errout << errmsg2 << std::endl;
 }
 
 void CppCheck::reportOut(const std::string &outmsg)
@@ -515,24 +514,45 @@ void CppCheck::reportOut(const std::string &outmsg)
     _errorLogger.reportOut(outmsg);
 }
 
-void CppCheck::reportProgress(const std::string &filename, const char stage[], const unsigned int value)
+void CppCheck::reportProgress(const std::string &filename, const char stage[], const std::size_t value)
 {
     _errorLogger.reportProgress(filename, stage, value);
 }
 
-void CppCheck::reportStatus(unsigned int /*fileindex*/, unsigned int /*filecount*/, long /*sizedone*/, long /*sizetotal*/)
+void CppCheck::reportInfo(const ErrorLogger::ErrorMessage &msg)
+{
+    // Suppressing info message?
+    std::string file;
+    unsigned int line(0);
+    if (!msg._callStack.empty()) {
+        file = msg._callStack.back().getfile(false);
+        line = msg._callStack.back().line;
+    }
+    if (_useGlobalSuppressions) {
+        if (_settings.nomsg.isSuppressed(msg._id, file, line))
+            return;
+    } else {
+        if (_settings.nomsg.isSuppressedLocal(msg._id, file, line))
+            return;
+    }
+
+    _errorLogger.reportInfo(msg);
+}
+
+void CppCheck::reportStatus(unsigned int /*fileindex*/, unsigned int /*filecount*/, std::size_t /*sizedone*/, std::size_t /*sizetotal*/)
 {
 
 }
 
 void CppCheck::getErrorMessages()
 {
+    tooManyConfigs = true;
+    tooManyConfigsError("",0U);
+
     // call all "getErrorMessages" in all registered Check classes
     for (std::list<Check *>::iterator it = Check::instances().begin(); it != Check::instances().end(); ++it)
         (*it)->getErrorMessages(this, &_settings);
 
-    Tokenizer tokenizer(&_settings, 0);
-    tokenizer.getErrorMessages(this, &_settings);
-
+    Tokenizer::getErrorMessages(this, &_settings);
     Preprocessor::getErrorMessages(this, &_settings);
 }
