@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -75,6 +75,7 @@ std::string Settings::addEnabled(const std::string &str)
 
     static std::set<std::string> id;
     if (id.empty()) {
+        id.insert("warning");
         id.insert("style");
         id.insert("performance");
         id.insert("portability");
@@ -147,6 +148,7 @@ bool Settings::platform(PlatformType type)
         sizeof_float = sizeof(float);
         sizeof_double = sizeof(double);
         sizeof_long_double = sizeof(long double);
+        sizeof_wchar_t = sizeof(wchar_t);
         sizeof_size_t = sizeof(std::size_t);
         sizeof_pointer = sizeof(void *);
         return true;
@@ -161,6 +163,7 @@ bool Settings::platform(PlatformType type)
         sizeof_float = 4;
         sizeof_double = 8;
         sizeof_long_double = 8;
+        sizeof_wchar_t = 2;
         sizeof_size_t = 4;
         sizeof_pointer = 4;
         return true;
@@ -174,6 +177,7 @@ bool Settings::platform(PlatformType type)
         sizeof_float = 4;
         sizeof_double = 8;
         sizeof_long_double = 8;
+        sizeof_wchar_t = 2;
         sizeof_size_t = 8;
         sizeof_pointer = 8;
         return true;
@@ -187,6 +191,7 @@ bool Settings::platform(PlatformType type)
         sizeof_float = 4;
         sizeof_double = 8;
         sizeof_long_double = 12;
+        sizeof_wchar_t = 4;
         sizeof_size_t = 4;
         sizeof_pointer = 4;
         return true;
@@ -200,6 +205,7 @@ bool Settings::platform(PlatformType type)
         sizeof_float = 4;
         sizeof_double = 8;
         sizeof_long_double = 16;
+        sizeof_wchar_t = 4;
         sizeof_size_t = 8;
         sizeof_pointer = 8;
         return true;
