@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -68,21 +68,17 @@ const Token* TemplateSimplifier::hasComplicatedSyntaxErrorsInTemplates(Token *to
 {
     // check for more complicated syntax errors when using templates..
     for (const Token *tok = tokens; tok; tok = tok->next()) {
+        // skip executing scopes (ticket #3183)..
+        if (Token::simpleMatch(tok, "( {"))
+            tok = tok->link();
+
         // skip executing scopes..
-        if (Token::simpleMatch(tok, ") {") || Token::Match(tok, ") %var% {") || Token::Match(tok, "[,=] {")) {
+        if (Token::simpleMatch(tok, ") {") || Token::Match(tok, ") %var% {") || Token::Match(tok, "[;,=] {")) {
             while (tok->str() != "{")
                 tok = tok->next();
             tok = tok->link();
         }
 
-        // skip executing scopes (ticket #1984)..
-        else if (Token::simpleMatch(tok, "; {"))
-            tok = tok->next()->link();
-
-        // skip executing scopes (ticket #3183)..
-        else if (Token::simpleMatch(tok, "( {"))
-            tok = tok->next()->link();
-
         // skip executing scopes (ticket #1985)..
         else if (Token::simpleMatch(tok, "try {")) {
             tok = tok->next()->link();
@@ -214,8 +210,10 @@ unsigned int TemplateSimplifier::templateParameters(const Token *tok)
             return 0;
 
         // Function pointer or prototype..
-        while (tok->str() == "(")
+        while (tok && tok->str() == "(")
             tok = tok->link()->next();
+        if (!tok)
+            return 0;
 
         // inner template
         if (tok->str() == "<") {
@@ -587,7 +585,7 @@ void TemplateSimplifier::expandTemplate(
     const std::string &name,
     std::vector<const Token *> &typeParametersInDeclaration,
     const std::string &newName,
-    std::vector<const Token *> &typesUsedInTemplateInstantion,
+    std::vector<const Token *> &typesUsedInTemplateInstantiation,
     std::list<Token *> &templateInstantiations)
 {
     for (const Token *tok3 = tokenlist.front(); tok3; tok3 = tok3->next()) {
@@ -622,7 +620,7 @@ void TemplateSimplifier::expandTemplate(
 
                 // replace type with given type..
                 if (itype < typeParametersInDeclaration.size()) {
-                    for (const Token *typetok = typesUsedInTemplateInstantion[itype];
+                    for (const Token *typetok = typesUsedInTemplateInstantiation[itype];
                          typetok && !Token::Match(typetok, "[,>]");
                          typetok = typetok->next()) {
                         tokenlist.addtoken(typetok, tok3->linenr(), tok3->fileIndex());
@@ -694,7 +692,7 @@ static bool isLowerThanAnd(const Token* lower)
 }
 static bool isLowerThanShift(const Token* lower)
 {
-    return isLowerThanAnd(lower) || Token::Match(lower, "&|<|<=|>|>=|==|!=");
+    return isLowerThanAnd(lower) || Token::Match(lower, "%comp%|&");
 }
 static bool isLowerThanPlusMinus(const Token* lower)
 {
@@ -791,11 +789,12 @@ bool TemplateSimplifier::simplifyCalculations(Token *_tokens)
         // keep parentheses here: operator new [] (size_t);
         // keep parentheses here: Functor()(a ... )
         // keep parentheses here: ) ( var ) ;
-        if (Token::Match(tok->next(), "( %var% ) ;|)|,|]|%op%") &&
+        if ((Token::Match(tok->next(), "( %var% ) ;|)|,|]") ||
+             (Token::Match(tok->next(), "( %var% ) %cop%") && (tok->tokAt(2)->varId()>0 || !Token::Match(tok->tokAt(4), "[*&]")))) &&
             !tok->isName() &&
             tok->str() != ">" &&
             tok->str() != "]" &&
-            !Token::simpleMatch(tok->previous(), "operator") &&
+            tok->strAt(-1) != "operator" &&
             !Token::simpleMatch(tok->previous(), "* )") &&
             !Token::simpleMatch(tok->previous(), ") )") &&
             !Token::Match(tok->tokAt(-2), "* %var% )") &&
@@ -808,8 +807,7 @@ bool TemplateSimplifier::simplifyCalculations(Token *_tokens)
             ret = true;
         }
 
-        if (tok->type() == Token::eChar &&
-            Token::Match(tok->previous(), "(|&&|%oror% %any% ==|!=|<=|<|>=|> %num% &&|%oror%|)")) {
+        if (Token::Match(tok->previous(), "(|&&|%oror% %char% %comp% %num% &&|%oror%|)")) {
             tok->str(MathLib::longToString(tok->str()[1] & 0xff));
         }
 
@@ -851,19 +849,19 @@ bool TemplateSimplifier::simplifyCalculations(Token *_tokens)
                     tok->deleteNext();
                     tok->deleteThis();
                     ret = true;
-                } else if (Token::Match(tok->previous(), "[=[(,] 0 * %var% ,|]|)|;|=|%op%") ||
-                           Token::Match(tok->previous(), "[=[(,] 0 * %num% ,|]|)|;|=|%op%") ||
-                           Token::Match(tok->previous(), "[=[(,] 0 * ( ,|]|)|;|=|%op%") ||
-                           Token::Match(tok->previous(), "return|case 0 *|&& %var% ,|:|;|=|%op%") ||
-                           Token::Match(tok->previous(), "return|case 0 *|&& %num% ,|:|;|=|%op%") ||
-                           Token::Match(tok->previous(), "return|case 0 *|&& ( ,|:|;|=|%op%")) {
+                } else if (Token::Match(tok->previous(), "[=[(,] 0 * %var% ,|]|)|;|=|%cop%") ||
+                           Token::Match(tok->previous(), "[=[(,] 0 * %num% ,|]|)|;|%op%") ||
+                           Token::Match(tok->previous(), "[=[(,] 0 * (") ||
+                           Token::Match(tok->previous(), "return|case 0 *|&& %var% ,|:|;|=|%cop%") ||
+                           Token::Match(tok->previous(), "return|case 0 *|&& %num% ,|:|;|%op%") ||
+                           Token::Match(tok->previous(), "return|case 0 *|&& (")) {
                     tok->deleteNext();
                     if (tok->next()->str() == "(")
                         Token::eraseTokens(tok, tok->next()->link());
                     tok->deleteNext();
                     ret = true;
-                } else if (Token::Match(tok->previous(), "[=[(,] 0 && *|& %any% ,|]|)|;|=|%op%") ||
-                           Token::Match(tok->previous(), "return|case 0 && *|& %any% ,|:|;|=|%op%")) {
+                } else if (Token::Match(tok->previous(), "[=[(,] 0 && *|& %any% ,|]|)|;|=|%cop%") ||
+                           Token::Match(tok->previous(), "return|case 0 && *|& %any% ,|:|;|=|%cop%")) {
                     tok->deleteNext();
                     tok->deleteNext();
                     if (tok->next()->str() == "(")
@@ -874,15 +872,15 @@ bool TemplateSimplifier::simplifyCalculations(Token *_tokens)
             }
 
             if (tok->str() == "1") {
-                if (Token::Match(tok->previous(), "[=[(,] 1 %oror% %any% ,|]|)|;|=|%op%") ||
-                    Token::Match(tok->previous(), "return|case 1 %oror% %any% ,|:|;|=|%op%")) {
+                if (Token::Match(tok->previous(), "[=[(,] 1 %oror% %any% ,|]|)|;|=|%cop%") ||
+                    Token::Match(tok->previous(), "return|case 1 %oror% %any% ,|:|;|=|%cop%")) {
                     tok->deleteNext();
                     if (tok->next()->str() == "(")
                         Token::eraseTokens(tok, tok->next()->link());
                     tok->deleteNext();
                     ret = true;
-                } else if (Token::Match(tok->previous(), "[=[(,] 1 %oror% *|& %any% ,|]|)|;|=|%op%") ||
-                           Token::Match(tok->previous(), "return|case 1 %oror% *|& %any% ,|:|;|=|%op%")) {
+                } else if (Token::Match(tok->previous(), "[=[(,] 1 %oror% *|& %any% ,|]|)|;|=|%cop%") ||
+                           Token::Match(tok->previous(), "return|case 1 %oror% *|& %any% ,|:|;|=|%cop%")) {
                     tok->deleteNext();
                     tok->deleteNext();
                     if (tok->next()->str() == "(")
@@ -893,7 +891,7 @@ bool TemplateSimplifier::simplifyCalculations(Token *_tokens)
             }
 
             if (Token::simpleMatch(tok->previous(), "* 1") || Token::simpleMatch(tok, "1 *")) {
-                if (tok->previous() && tok->previous()->isOp())
+                if (tok->previous() && tok->previous()->isConstOp())
                     tok = tok->previous();
                 tok->deleteNext();
                 tok->deleteThis();
@@ -914,14 +912,14 @@ bool TemplateSimplifier::simplifyCalculations(Token *_tokens)
                 Token::simpleMatch(tok->previous(), "| 0 )") ||
                 Token::simpleMatch(tok->previous(), "( 1 &&") ||
                 Token::simpleMatch(tok->previous(), "&& 1 )")) {
-                if (tok->previous()->isOp())
+                if (tok->previous()->isConstOp())
                     tok = tok->previous();
                 tok->deleteNext();
                 tok->deleteThis();
                 ret = true;
             }
 
-            if (Token::Match(tok, "%num% ==|!=|<=|>=|<|> %num%") &&
+            if (Token::Match(tok, "%num% %comp% %num%") &&
                 MathLib::isInt(tok->str()) &&
                 MathLib::isInt(tok->strAt(2))) {
                 if (Token::Match(tok->previous(), "(|&&|%oror%") && Token::Match(tok->tokAt(3), ")|&&|%oror%")) {
@@ -964,7 +962,7 @@ bool TemplateSimplifier::simplifyCalculations(Token *_tokens)
 }
 
 
-bool TemplateSimplifier::simplifyTemplateInstantions(
+bool TemplateSimplifier::simplifyTemplateInstantiations(
     TokenList& tokenlist,
     ErrorLogger& errorlogger,
     const Settings *_settings,
@@ -1029,12 +1027,12 @@ bool TemplateSimplifier::simplifyTemplateInstantions(
             continue;
 
         // New type..
-        std::vector<const Token *> typesUsedInTemplateInstantion;
+        std::vector<const Token *> typesUsedInTemplateInstantiation;
         std::string typeForNewNameStr;
         std::string templateMatchPattern(name + " < ");
         unsigned int indentlevel = 0;
         for (const Token *tok3 = tok2->tokAt(2); tok3 && (indentlevel > 0 || tok3->str() != ">"); tok3 = tok3->next()) {
-            // #2648 - unhandled parenthesis => bail out
+            // #2648 - unhandled parentheses => bail out
             // #2721 - unhandled [ => bail out
             if (tok3->str() == "(" || tok3->str() == "[") {
                 typeForNewNameStr.clear();
@@ -1051,7 +1049,7 @@ bool TemplateSimplifier::simplifyTemplateInstantions(
             templateMatchPattern += tok3->str();
             templateMatchPattern += " ";
             if (indentlevel == 0 && Token::Match(tok3->previous(), "[<,]"))
-                typesUsedInTemplateInstantion.push_back(tok3);
+                typesUsedInTemplateInstantiation.push_back(tok3);
             // add additional type information
             if (tok3->isUnsigned())
                 typeForNewNameStr += "unsigned";
@@ -1064,7 +1062,7 @@ bool TemplateSimplifier::simplifyTemplateInstantions(
         templateMatchPattern += ">";
         const std::string typeForNewName(typeForNewNameStr);
 
-        if (typeForNewName.empty() || typeParametersInDeclaration.size() != typesUsedInTemplateInstantion.size()) {
+        if (typeForNewName.empty() || typeParametersInDeclaration.size() != typesUsedInTemplateInstantiation.size()) {
             if (_settings->debugwarnings) {
                 std::list<const Token *> callstack(1, tok);
                 errorlogger.reportErr(ErrorLogger::ErrorMessage(callstack, &tokenlist, Severity::debug, "debg",
@@ -1080,7 +1078,7 @@ bool TemplateSimplifier::simplifyTemplateInstantions(
 
         if (expandedtemplates.find(newName) == expandedtemplates.end()) {
             expandedtemplates.insert(newName);
-            TemplateSimplifier::expandTemplate(tokenlist, tok,name,typeParametersInDeclaration,newName,typesUsedInTemplateInstantion,templateInstantiations);
+            TemplateSimplifier::expandTemplate(tokenlist, tok,name,typeParametersInDeclaration,newName,typesUsedInTemplateInstantiation,templateInstantiations);
             instantiated = true;
         }
 
@@ -1089,8 +1087,8 @@ bool TemplateSimplifier::simplifyTemplateInstantions(
         for (Token *tok4 = tok2; tok4; tok4 = tok4->next()) {
             if (Token::simpleMatch(tok4, templateMatchPattern.c_str())) {
                 Token * tok5 = tok4->tokAt(2);
-                unsigned int typeCountInInstantion = 1U; // There is always atleast one type
-                const Token *typetok = (!typesUsedInTemplateInstantion.empty()) ? typesUsedInTemplateInstantion[0] : 0;
+                unsigned int typeCountInInstantiation = 1U; // There is always at least one type
+                const Token *typetok = (!typesUsedInTemplateInstantiation.empty()) ? typesUsedInTemplateInstantiation[0] : 0;
                 unsigned int indentlevel5 = 0;  // indentlevel for tok5
                 while (tok5 && (indentlevel5 > 0 || tok5->str() != ">")) {
                     if (tok5->str() == "<" && templateParameters(tok5) > 0)
@@ -1108,8 +1106,8 @@ bool TemplateSimplifier::simplifyTemplateInstantions(
 
                             typetok = typetok ? typetok->next() : 0;
                         } else {
-                            typetok = (typeCountInInstantion < typesUsedInTemplateInstantion.size()) ? typesUsedInTemplateInstantion[typeCountInInstantion] : 0;
-                            ++typeCountInInstantion;
+                            typetok = (typeCountInInstantiation < typesUsedInTemplateInstantiation.size()) ? typesUsedInTemplateInstantiation[typeCountInInstantiation] : 0;
+                            ++typeCountInInstantiation;
                         }
                     }
                     tok5 = tok5->next();
@@ -1117,7 +1115,7 @@ bool TemplateSimplifier::simplifyTemplateInstantions(
 
                 // matching template usage => replace tokens..
                 // Foo < int >  =>  Foo<int>
-                if (tok5 && tok5->str() == ">" && typeCountInInstantion == typesUsedInTemplateInstantion.size()) {
+                if (tok5 && tok5->str() == ">" && typeCountInInstantiation == typesUsedInTemplateInstantiation.size()) {
                     tok4->str(newName);
                     for (Token *tok6 = tok4->next(); tok6 != tok5; tok6 = tok6->next()) {
                         if (tok6->isName())
@@ -1190,7 +1188,7 @@ void TemplateSimplifier::simplifyTemplates(
         //done = true;
         std::list<Token *> templates2;
         for (std::list<Token *>::reverse_iterator iter1 = templates.rbegin(); iter1 != templates.rend(); ++iter1) {
-            bool instantiated = TemplateSimplifier::simplifyTemplateInstantions(tokenlist,
+            bool instantiated = TemplateSimplifier::simplifyTemplateInstantiations(tokenlist,
                                 errorlogger,
                                 _settings,
                                 *iter1,
