@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -84,13 +84,18 @@ public:
      *
      * @param FileName The filename
      * @param configuration E.g. "A" for code where "#ifdef A" is true
-     * @param preprocessorCondition Set this flag to true if the code is a preprocessor condition. It disables some simplifications
      * @return false if source code contains syntax errors
      */
     bool tokenize(std::istream &code,
                   const char FileName[],
-                  const std::string &configuration = "",
-                  const bool preprocessorCondition = false);
+                  const std::string &configuration = "");
+
+    /**
+     * tokenize condition and run simple simplifications on it
+     * @param code code
+     * @return true if success.
+     */
+    bool tokenizeCondition(const std::string &code);
 
     /** Set variable id */
     void setVarId();
@@ -172,6 +177,9 @@ public:
     /** Remove macros in global scope */
     void removeMacrosInGlobalScope();
 
+    /** Remove unknown macro in variable declarations: PROGMEM char x; */
+    void removeMacroInVarDecl();
+
     /** Remove redundant assignment */
     void removeRedundantAssignment();
 
@@ -273,15 +281,26 @@ public:
      */
     void simplifyComma();
 
-    /** Add braces to an if-block
+    /** Add braces to an if-block, for-block, etc.
      * @return true if no syntax errors
      */
-    bool simplifyIfAddBraces();
+    bool simplifyAddBraces();
 
-    /**
-     * Add braces to an do-while block
+    /** Add braces to an if-block, for-block, etc.
+     * for command starting at token including else-block
+     * @return last token of command
+     *         or input token in case of an error where no braces are added
+     *         or NULL when syntaxError is called
      */
-    void simplifyDoWhileAddBraces();
+    Token * simplifyAddBracesToCommand(Token * tok);
+
+    /** Add pair of braces to an single if-block, else-block, for-block, etc.
+     * for command starting at token
+     * @return last token of command
+     *         or input token in case of an error where no braces are added
+     *         or NULL when syntaxError is called
+     */
+    Token * simplifyAddBracesPair(Token *tok, bool commandWithCondition);
 
     /**
      * typedef A mytype;
@@ -387,7 +406,7 @@ public:
     void simplifyStructDecl();
 
     /**
-     * Remove redundant parenthesis:
+     * Remove redundant parentheses:
      * - "((x))" => "(x)"
      * - "(function())" => "function()"
      * - "(delete x)" => "delete x"
@@ -395,7 +414,9 @@ public:
      * @return true if modifications to token-list are done.
      *         false if no modifications are done.
      */
-    bool simplifyRedundantParenthesis();
+    bool simplifyRedundantParentheses();
+
+    void simplifyCharAt();
 
     /** Simplify references */
     void simplifyReference();
@@ -420,17 +441,23 @@ public:
 
     void simplifyParameterVoid();
 
+    void fillTypeSizes();
+
+    void combineOperators();
+
+    void combineStrings();
+
     void concatenateDoubleSharp();
 
-    void simplifyLineMacro();
+    void simplifyFileAndLineMacro();
 
     void simplifyNull();
 
-    void concatenateNegativeNumber();
+    void concatenateNegativeNumberAndAnyPositive();
 
     void simplifyExternC();
 
-    void simplifyRoundCurlyParenthesis();
+    void simplifyRoundCurlyParentheses();
 
     void simplifyDebugNew();
 
@@ -533,6 +560,9 @@ public:
     /** Syntax error. Example: invalid number of ')' */
     void syntaxError(const Token *tok, char c) const;
 
+    /** Report that there is an unhandled "class x y {" code */
+    void unhandled_macro_class_x_y(const Token *tok);
+
     /**
      * assert that tokens are ok - used during debugging for example
      * to catch problems in simplifyTokenList.
