@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -411,7 +411,7 @@ private:
             ASSERT_EQUALS(Settings::C, settings.enforcedLang);
         }
         {
-            const char *argv[] = {"cppcheck", "--language=unknwonLanguage", "file.cpp"};
+            const char *argv[] = {"cppcheck", "--language=unknownLanguage", "file.cpp"};
             Settings settings;
             CmdLineParser parser(&settings);
             ASSERT(!parser.ParseFromArgs(3, argv));
