@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -53,6 +53,7 @@ private:
         errout.str("");
 
         Settings settings;
+        settings.addEnabled("warning");
         settings.addEnabled("style");
         if (portability)
             settings.addEnabled("portability");
@@ -287,6 +288,66 @@ private:
               "    fclose(f[0]);\n"
               "}");
         ASSERT_EQUALS("", errout.str());
+
+        // #4368: multiple functions
+        check("static FILE *fp = NULL;\n"
+              "\n"
+              "void close()\n"
+              "{\n"
+              "  fclose(fp);\n"
+              "}\n"
+              "\n"
+              "void dump()\n"
+              "{\n"
+              "  if (fp == NULL) return;\n"
+              "  fprintf(fp, \"Here's the output.\\n\");\n"
+              "}\n"
+              "\n"
+              "int main()\n"
+              "{\n"
+              "  fp = fopen(\"test.txt\", \"w\");\n"
+              "  dump();\n"
+              "  close();\n"
+              "  return 0;\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        check("static FILE *fp = NULL;\n"
+              "\n"
+              "void close()\n"
+              "{\n"
+              "  fclose(fp);\n"
+              "}\n"
+              "\n"
+              "void dump()\n"
+              "{\n"
+              "  fclose(fp);\n"
+              "  fprintf(fp, \"Here's the output.\\n\");\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:11]: (error) Used file that is not opened.\n", errout.str());
+
+        // #4466
+        check("void chdcd_parse_nero(FILE *infile) {\n"
+              "    switch (mode) {\n"
+              "        case 0x0300:\n"
+              "            fclose(infile);\n"
+              "            return;\n"
+              "        case 0x0500:\n"
+              "            fclose(infile);\n"
+              "            return;\n"
+              "    }\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        // #4649
+        check("void foo() {\n"
+              "    struct {FILE *f1; FILE *f2;} a;\n"
+              "    a.f1 = fopen(name,mode);\n"
+              "    a.f2 = fopen(name,mode);\n"
+              "    fclose(a.f1);\n"
+              "    fclose(a.f2);\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
     }
 
     void fileIOwithoutPositioning() {
@@ -413,7 +474,7 @@ private:
               "{\n"
               "  char str [8];\n"
               "  scanf (\"%70s\",str);\n"
-              "}\n");
+              "}");
         ASSERT_EQUALS("[test.cpp:4]: (error) Width 70 given in format string (no. 1) is larger than destination buffer 'str[8]', use %7s to prevent overflowing it.\n", errout.str());
     }
 
@@ -637,9 +698,9 @@ private:
               "    printf(\"%f\", f);\n"
               "    printf(\"%p\", f);\n"
               "}");
-        TODO_ASSERT_EQUALS("[test.cpp:3]: (warning) %u in format string (no. 1) requires an integer given in the argument list.\n"
-                           "[test.cpp:4]: (warning) %f in format string (no. 1) requires an integer given in the argument list.\n"
-                           "[test.cpp:5]: (warning) %p in format string (no. 1) requires an integer given in the argument list.\n", "", errout.str());
+        ASSERT_EQUALS("[test.cpp:3]: (warning) %u in format string (no. 1) requires an unsigned integer given in the argument list.\n"
+                      "[test.cpp:4]: (warning) %f in format string (no. 1) requires a floating point number given in the argument list.\n"
+                      "[test.cpp:5]: (warning) %p in format string (no. 1) requires an address given in the argument list.\n", errout.str());
 
         // Ticket #4189 (Improve check (printf("%l") not detected)) tests (according to C99 7.19.6.1.7)
         // False positive tests
