@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -62,6 +62,36 @@ private:
         found = false;
     }
 
+    static const Scope *findFunctionScopeByToken(const SymbolDatabase * db, const Token *tok) {
+        std::list<Scope>::const_iterator scope;
+
+        for (scope = db->scopeList.begin(); scope != db->scopeList.end(); ++scope) {
+            if (scope->type == Scope::eFunction) {
+                if (scope->classDef == tok)
+                    return &(*scope);
+            }
+        }
+        return 0;
+    }
+
+    static const Function *findFunctionByName(const std::string& str, const Scope* startScope) {
+        const Scope* currScope = startScope;
+        while (currScope && currScope->isExecutable()) {
+            if (currScope->functionOf)
+                currScope = currScope->functionOf;
+            else
+                currScope = currScope->nestedIn;
+        }
+        while (currScope) {
+            for (std::list<Function>::const_iterator i = currScope->functionList.begin(); i != currScope->functionList.end(); ++i) {
+                if (i->tokenDef->str() == str)
+                    return &*i;
+            }
+            currScope = currScope->nestedIn;
+        }
+        return 0;
+    }
+
     void run() {
         TEST_CASE(array);
 
@@ -119,6 +149,7 @@ private:
         TEST_CASE(functionArgs1);
         TEST_CASE(functionArgs2);
         TEST_CASE(functionArgs3);
+        TEST_CASE(functionArgs4);
 
         TEST_CASE(namespaces1);
         TEST_CASE(namespaces2);
@@ -154,6 +185,10 @@ private:
         TEST_CASE(symboldatabase26); // ticket #3561 (throw C)
         TEST_CASE(symboldatabase27); // ticket #3543 (segmentation fault)
         TEST_CASE(symboldatabase28);
+        TEST_CASE(symboldatabase29); // ticket #4442 (segmentation fault)
+        TEST_CASE(symboldatabase30);
+        TEST_CASE(symboldatabase31);
+        TEST_CASE(symboldatabase33); // ticket #4682 (false negatives)
 
         TEST_CASE(isImplicitlyVirtual);
 
@@ -162,7 +197,7 @@ private:
         TEST_CASE(findFunction1);
     }
 
-    void array() {
+    void array() const {
         std::istringstream code("int a[10+2];");
         TokenList list(NULL);
         list.createTokens(code, "test.c");
@@ -510,12 +545,12 @@ private:
         ASSERT(db && db->scopeList.size() == 2);
 
         if (db) {
-            const Scope *scope = db->findFunctionScopeByToken(tokenizer.tokens()->next());
+            const Scope *scope = findFunctionScopeByToken(db, tokenizer.tokens()->next());
 
             ASSERT(scope && scope->className == "func");
             ASSERT(scope && scope->functionOf == 0);
 
-            const Function *function = db->findFunctionByName("func", &db->scopeList.front());
+            const Function *function = findFunctionByName("func", &db->scopeList.front());
 
             ASSERT(function && function->token->str() == "func");
             ASSERT(function && function->token == tokenizer.tokens()->next());
@@ -531,17 +566,19 @@ private:
         ASSERT(db && db->scopeList.size() == 3);
 
         if (db) {
-            const Scope *scope = db->findFunctionScopeByToken(tokenizer.tokens()->tokAt(4));
+            const Scope *scope = findFunctionScopeByToken(db, tokenizer.tokens()->tokAt(4));
 
             ASSERT(scope && scope->className == "func");
             ASSERT(scope && scope->functionOf && scope->functionOf == db->findScopeByName("Fred"));
 
-            const Function *function = db->findFunctionByName("func", &db->scopeList.back());
+            const Function *function = findFunctionByName("func", &db->scopeList.back());
 
             ASSERT(function && function->token->str() == "func");
             ASSERT(function && function->token == tokenizer.tokens()->tokAt(4));
             ASSERT(function && function->hasBody && function->isInline);
             ASSERT(function && function->functionScope == scope && scope->function == function && function->nestedIn == db->findScopeByName("Fred"));
+
+            ASSERT(db && db->findScopeByName("Fred") && db->findScopeByName("Fred")->definedType->getFunction("func") == function);
         }
     }
 
@@ -552,11 +589,11 @@ private:
         ASSERT(db && db->scopeList.size() == 2);
 
         if (db) {
-            const Scope *scope = db->findFunctionScopeByToken(tokenizer.tokens()->tokAt(4));
+            const Scope *scope = findFunctionScopeByToken(db, tokenizer.tokens()->tokAt(4));
 
             ASSERT(scope == NULL);
 
-            const Function *function = db->findFunctionByName("func", &db->scopeList.back());
+            const Function *function = findFunctionByName("func", &db->scopeList.back());
 
             ASSERT(function && function->token->str() == "func");
             ASSERT(function && function->token == tokenizer.tokens()->tokAt(4));
@@ -571,12 +608,12 @@ private:
         ASSERT(db && db->scopeList.size() == 3);
 
         if (db) {
-            const Scope *scope = db->findFunctionScopeByToken(tokenizer.tokens()->tokAt(12));
+            const Scope *scope = findFunctionScopeByToken(db, tokenizer.tokens()->tokAt(12));
 
             ASSERT(scope && scope->className == "func");
             ASSERT(scope && scope->functionOf && scope->functionOf == db->findScopeByName("Fred"));
 
-            const Function *function = db->findFunctionByName("func", &db->scopeList.back());
+            const Function *function = findFunctionByName("func", &db->scopeList.back());
 
             ASSERT(function && function->token->str() == "func");
             ASSERT(function && function->token == tokenizer.tokens()->tokAt(12));
@@ -592,11 +629,11 @@ private:
         ASSERT(db && db->scopeList.size() == 2);
 
         if (db) {
-            const Scope *scope = db->findFunctionScopeByToken(tokenizer.tokens()->tokAt(3));
+            const Scope *scope = findFunctionScopeByToken(db, tokenizer.tokens()->tokAt(3));
 
             ASSERT(scope && scope->className == "func");
 
-            const Function *function = db->findFunctionByName("func", &db->scopeList.front());
+            const Function *function = findFunctionByName("func", &db->scopeList.front());
 
             ASSERT(function && function->token->str() == "func");
             ASSERT(function && function->token == tokenizer.tokens()->tokAt(3));
@@ -611,11 +648,11 @@ private:
         ASSERT(db && db->scopeList.size() == 3);
 
         if (db) {
-            const Scope *scope = db->findFunctionScopeByToken(tokenizer.tokens()->tokAt(6));
+            const Scope *scope = findFunctionScopeByToken(db, tokenizer.tokens()->tokAt(6));
 
             ASSERT(scope && scope->className == "func");
 
-            const Function *function = db->findFunctionByName("func", &db->scopeList.back());
+            const Function *function = findFunctionByName("func", &db->scopeList.back());
 
             ASSERT(function && function->token->str() == "func");
             ASSERT(function && function->token == tokenizer.tokens()->tokAt(6));
@@ -630,11 +667,11 @@ private:
         ASSERT(db && db->scopeList.size() == 2);
 
         if (db) {
-            const Scope *scope = db->findFunctionScopeByToken(tokenizer.tokens()->tokAt(6));
+            const Scope *scope = findFunctionScopeByToken(db, tokenizer.tokens()->tokAt(6));
 
             ASSERT(scope == NULL);
 
-            const Function *function = db->findFunctionByName("func", &db->scopeList.back());
+            const Function *function = findFunctionByName("func", &db->scopeList.back());
 
             ASSERT(function && function->token->str() == "func");
             ASSERT(function && function->token == tokenizer.tokens()->tokAt(6));
@@ -649,11 +686,11 @@ private:
         ASSERT(db && db->scopeList.size() == 3);
 
         if (db) {
-            const Scope *scope = db->findFunctionScopeByToken(tokenizer.tokens()->tokAt(23));
+            const Scope *scope = findFunctionScopeByToken(db, tokenizer.tokens()->tokAt(23));
 
             ASSERT(scope && scope->className == "func");
 
-            const Function *function = db->findFunctionByName("func", &db->scopeList.back());
+            const Function *function = findFunctionByName("func", &db->scopeList.back());
 
             ASSERT(function && function->token->str() == "func");
             ASSERT(function && function->token == tokenizer.tokens()->tokAt(23));
@@ -675,7 +712,7 @@ private:
                     seen_something = true;
                 }
             }
-            TODO_ASSERT_EQUALS("works", "doesn't work", seen_something ? "works" : "doesn't work");
+            ASSERT_EQUALS(true, seen_something);
         }
     }
 
@@ -683,7 +720,7 @@ private:
         GET_SYMBOL_DB("std::map<int, string> foo() {}")
 
         // 2 scopes: Global and Function
-        ASSERT(db && db->scopeList.size() == 2 && db->findFunctionByName("foo", &db->scopeList.back()));
+        ASSERT(db && db->scopeList.size() == 2 && findFunctionByName("foo", &db->scopeList.back()));
 
         if (db) {
             const Scope *scope = &db->scopeList.front();
@@ -701,7 +738,7 @@ private:
         GET_SYMBOL_DB("void foo();\nvoid foo();\nint foo(int i);\nvoid foo() {}")
 
         // 2 scopes: Global and Function
-        ASSERT(db && db->scopeList.size() == 2 && db->findFunctionByName("foo", &db->scopeList.back()));
+        ASSERT(db && db->scopeList.size() == 2 && findFunctionByName("foo", &db->scopeList.back()));
 
         if (db) {
             const Scope *scope = &db->scopeList.front();
@@ -735,8 +772,8 @@ private:
             ASSERT(bar2 != 0);
 
             if (foo && bar1 && bar2) {
-                ASSERT(bar1->friendList.size() == 1 && bar1->friendList.front().name == "Foo" && bar1->friendList.front().scope == foo);
-                ASSERT(bar2->friendList.size() == 1 && bar2->friendList.front().name == "Foo" && bar2->friendList.front().scope == foo);
+                ASSERT(bar1->definedType->friendList.size() == 1 && bar1->definedType->friendList.front().name == "Foo" && bar1->definedType->friendList.front().type == foo->definedType);
+                ASSERT(bar2->definedType->friendList.size() == 1 && bar2->definedType->friendList.front().name == "Foo" && bar2->definedType->friendList.front().type == foo->definedType);
             }
         }
     }
@@ -749,8 +786,8 @@ private:
         // 3 scopes: Global, function, if
         ASSERT_EQUALS(3, db->scopeList.size());
 
-        ASSERT(db->findFunctionByName("func", &db->scopeList.back()) != NULL);
-        ASSERT(db->findFunctionByName("if", &db->scopeList.back()) == NULL);
+        ASSERT(findFunctionByName("func", &db->scopeList.back()) != NULL);
+        ASSERT(findFunctionByName("if", &db->scopeList.back()) == NULL);
     }
 
     void parseFunctionDeclarationCorrect() {
@@ -907,13 +944,30 @@ private:
         ASSERT_EQUALS("i", a->nameToken()->str());
     }
 
+    void functionArgs4() {
+        GET_SYMBOL_DB("void f1(char [10], struct foo [10]);");
+        ASSERT_EQUALS(true, db->scopeList.front().functionList.size() == 1UL);
+        const Function *func = &db->scopeList.front().functionList.front();
+        ASSERT_EQUALS(true, func && func->argumentList.size() == 2UL);
+        if (func && func->argumentList.size() == 2UL) {
+            const Variable *first = &func->argumentList.front();
+            ASSERT_EQUALS(0UL, first->name().size());
+            ASSERT_EQUALS(1UL, first->dimensions().size());
+            ASSERT_EQUALS(10UL, first->dimension(0));
+            const Variable *second = &func->argumentList.back();
+            ASSERT_EQUALS(0UL, second->name().size());
+            ASSERT_EQUALS(1UL, second->dimensions().size());
+            ASSERT_EQUALS(10UL, second->dimension(0));
+        }
+    }
+
     void namespaces1() {
         GET_SYMBOL_DB("namespace fred {\n"
                       "    namespace barney {\n"
                       "        class X { X(int); };\n"
                       "    }\n"
                       "}\n"
-                      "namespace barney { X::X(int) { } }\n");
+                      "namespace barney { X::X(int) { } }");
 
         // Locate the scope for the class..
         const Scope *scope = NULL;
@@ -947,7 +1001,7 @@ private:
                       "    namespace barney {\n"
                       "        X::X(int) { }\n"
                       "    }\n"
-                      "}\n");
+                      "}");
 
         // Locate the scope for the class..
         const Scope *scope = NULL;
@@ -973,7 +1027,7 @@ private:
 
     void namespaces3() { // #3854 - namespace with unknown macro
         GET_SYMBOL_DB("namespace fred UNKNOWN_MACRO(default) {\n"
-                      "}\n");
+                      "}");
         ASSERT_EQUALS(2U, db->scopeList.size());
         ASSERT_EQUALS(Scope::eGlobal, db->scopeList.front().type);
         ASSERT_EQUALS(Scope::eNamespace, db->scopeList.back().type);
@@ -986,7 +1040,7 @@ private:
                               "    catch (const X::Error2 & x) { }\n"
                               "    catch (Error3 x) { }\n"
                               "    catch (X::Error4 x) { }\n"
-                              "}\n");
+                              "}");
         GET_SYMBOL_DB(str.c_str())
         check(str.c_str(), false);
         ASSERT_EQUALS("", errout.str());
@@ -1024,7 +1078,7 @@ private:
         check("typedef void (func_type)();\n"
               "struct A {\n"
               "    friend func_type f : 2;\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("", errout.str());
     }
 
@@ -1035,7 +1089,7 @@ private:
               "static void function_declaration_after(void) __attribute__((__used__));\n");
         ASSERT_EQUALS("", errout.str());
 
-        check("main(int argc, char *argv[]) { }\n");
+        check("main(int argc, char *argv[]) { }");
         ASSERT_EQUALS("", errout.str());
 
         check("namespace boost {\n"
@@ -1043,7 +1097,7 @@ private:
               "    {\n"
               "        return std::locale();\n"
               "    }\n"
-              "}\n");
+              "}");
         ASSERT_EQUALS("", errout.str());
 
         check("namespace X {\n"
@@ -1051,12 +1105,12 @@ private:
               "    static void function_declaration_before(void) {}\n"
               "    static void function_declaration_after(void) {}\n"
               "    static void function_declaration_after(void) __attribute__((__used__));\n"
-              "}\n");
+              "}");
         ASSERT_EQUALS("", errout.str());
 
         check("testing::testing()\n"
               "{\n"
-              "}\n");
+              "}");
         ASSERT_EQUALS("", errout.str());
     }
 
@@ -1064,7 +1118,7 @@ private:
         // ticket #2178 - segmentation fault
         check("int CL_INLINE_DECL(integer_decode_float) (int x) {\n"
               "    return (sign ? cl_I() : 0);\n"
-              "}\n");
+              "}");
         ASSERT_EQUALS("", errout.str());
     }
 
@@ -1091,7 +1145,7 @@ private:
               "{\n"
               "public:\n"
               "    int f() { return C< ::D,int>::f(); }\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("", errout.str());
     }
 
@@ -1104,7 +1158,7 @@ private:
               "    {\n"
               "        return vcl::unotools::createStandardColorSpace();\n"
               "    }\n"
-              "};\n");
+              "};");
 
         ASSERT_EQUALS("", errout.str());
     }
@@ -1131,7 +1185,7 @@ private:
               "};\n"
               "class B {\n"
               "  friend void A::f();\n"
-              "};\n");
+              "};");
 
         ASSERT_EQUALS("", errout.str());
     }
@@ -1141,7 +1195,7 @@ private:
         check("int g ();\n"
               "struct S {\n"
               "  int i : (false ? g () : 1);\n"
-              "};\n");
+              "};");
 
         ASSERT_EQUALS("", errout.str());
     }
@@ -1150,7 +1204,7 @@ private:
         // ticket #2547 - segmentation fault
         check("class foo {\n"
               "    void bar2 () = __null;\n"
-              "};\n");
+              "};");
 
         ASSERT_EQUALS("", errout.str());
     }
@@ -1159,7 +1213,7 @@ private:
         // ticket #2577 - segmentation fault
         check("class foo {\n"
               "    void bar2 () = A::f;\n"
-              "};\n");
+              "};");
 
         ASSERT_EQUALS("", errout.str());
     }
@@ -1187,7 +1241,7 @@ private:
 
     void symboldatabase17() {
         // ticket #2657 - segmentation fault
-        check("return f(){}\n");
+        check("return f(){}");
 
         ASSERT_EQUALS("", errout.str());
     }
@@ -1201,7 +1255,7 @@ private:
 
     void symboldatabase19() {
         // ticket #2991 - segmentation fault
-        check("::y(){x}\n");
+        check("::y(){x}");
 
         ASSERT_EQUALS("", errout.str());
     }
@@ -1220,7 +1274,7 @@ private:
               "};\n"
               "Fred::func() const {\n"
               "    Foo foo;\n"
-              "}\n");
+              "}");
 
         ASSERT_EQUALS("", errout.str());
     }
@@ -1234,7 +1288,7 @@ private:
 
     // #ticket 3435 (std::vector)
     void symboldatabase23() {
-        GET_SYMBOL_DB("class A { std::vector<int*> ints; };\n");
+        GET_SYMBOL_DB("class A { std::vector<int*> ints; };");
         ASSERT_EQUALS(2U, db->scopeList.size());
         const Scope &scope = db->scopeList.back();
         ASSERT_EQUALS(1U, scope.varlist.size());
@@ -1314,14 +1368,92 @@ private:
               "{\n"
               "    B1()\n"
               "    {} C(int) : B1() class\n"
-              "};\n");
+              "};");
         ASSERT_EQUALS("", errout.str());
     }
 
     void symboldatabase28() {
         GET_SYMBOL_DB("struct S {};\n"
                       "void foo(struct S s) {}");
-        ASSERT(db && db->getVariableFromVarId(1) && db->getVariableFromVarId(1)->type() && db->getVariableFromVarId(1)->type()->className == "S");
+        ASSERT(db && db->getVariableFromVarId(1) && db->getVariableFromVarId(1)->typeScope() && db->getVariableFromVarId(1)->typeScope()->className == "S");
+    }
+
+    // #ticket #4442 (segmentation fault)
+    void symboldatabase29() {
+        check("struct B : A {\n"
+              "    B() : A {}\n"
+              "};");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void symboldatabase30() {
+        GET_SYMBOL_DB("struct A { void foo(const int a); };\n"
+                      "void A::foo(int a) { }");
+        ASSERT(db && db->functionScopes.size() == 1 && db->functionScopes[0]->functionOf);
+    }
+
+    void symboldatabase31() {
+        GET_SYMBOL_DB("class Foo;\n"
+                      "class Bar;\n"
+                      "class Sub;\n"
+                      "class Foo { class Sub; };\n"
+                      "class Bar { class Sub; };\n"
+                      "class Bar::Sub {\n"
+                      "    int b;\n"
+                      "public:\n"
+                      "    Sub() { }\n"
+                      "    Sub(int);\n"
+                      "};\n"
+                      "Bar::Sub::Sub(int) { };\n"
+                      "class ::Foo::Sub {\n"
+                      "    int f;\n"
+                      "public:\n"
+                      "    ~Sub();\n"
+                      "    Sub();\n"
+                      "};\n"
+                      "::Foo::Sub::~Sub() { }\n"
+                      "::Foo::Sub::Sub() { }\n"
+                      "class Foo;\n"
+                      "class Bar;\n"
+                      "class Sub;\n");
+        ASSERT(db && db->typeList.size() == 5);
+        ASSERT(db && db->isClassOrStruct("Foo"));
+        ASSERT(db && db->isClassOrStruct("Bar"));
+        ASSERT(db && db->isClassOrStruct("Sub"));
+        if (!db || db->typeList.size() < 5)
+            return;
+        std::list<Type>::const_iterator i = db->typeList.begin();
+        const Type* Foo = &(*i++);
+        const Type* Bar = &(*i++);
+        const Type* Sub = &(*i++);
+        const Type* Foo_Sub = &(*i++);
+        const Type* Bar_Sub = &(*i);
+        ASSERT(Foo && Foo->classDef && Foo->classScope && Foo->enclosingScope && Foo->name() == "Foo");
+        ASSERT(Bar && Bar->classDef && Bar->classScope && Bar->enclosingScope && Bar->name() == "Bar");
+        ASSERT(Sub && Sub->classDef && !Sub->classScope && Sub->enclosingScope && Sub->name() == "Sub");
+        ASSERT(Foo_Sub && Foo_Sub->classDef && Foo_Sub->classScope && Foo_Sub->enclosingScope == Foo->classScope && Foo_Sub->name() == "Sub");
+        ASSERT(Bar_Sub && Bar_Sub->classDef && Bar_Sub->classScope && Bar_Sub->enclosingScope == Bar->classScope && Bar_Sub->name() == "Sub");
+        ASSERT(Foo_Sub && Foo_Sub->classScope && Foo_Sub->classScope->numConstructors == 1 && Foo_Sub->classScope->className == "Sub");
+        ASSERT(Bar_Sub && Bar_Sub->classScope && Bar_Sub->classScope->numConstructors == 2 && Bar_Sub->classScope->className == "Sub");
+    }
+
+    void symboldatabase32() {
+        GET_SYMBOL_DB("struct Base {\n"
+                      "    void foo() {}\n"
+                      "};\n"
+                      "class Deri : Base {\n"
+                      "};");
+        ASSERT(db && db->findScopeByName("Deri") && db->findScopeByName("Deri")->definedType->getFunction("foo"));
+    }
+
+    void symboldatabase33() { // ticket #4682
+        GET_SYMBOL_DB("static struct A::B s;\n"
+                      "static struct A::B t = { 0 };\n"
+                      "static struct A::B u(0);\n"
+                      "static struct A::B v{0};\n"
+                      "static struct A::B w({0});\n"
+                      "void foo() { }");
+        ASSERT(db && db->functionScopes.size() == 1);
     }
 
     void isImplicitlyVirtual() {
@@ -1426,7 +1558,7 @@ private:
                 unsigned int index = 0;
                 for (const Token * tok = bar->classStart->next(); tok != bar->classEnd; tok = tok->next()) {
                     if (Token::Match(tok, "%var% (") && !tok->varId() && Token::simpleMatch(tok->linkAt(1), ") ;")) {
-                        const Function * function = db->findFunctionByNameAndArgs(tok, bar);
+                        const Function * function = db->findFunction(tok);
                         ASSERT(function != 0);
                         if (function) {
                             std::stringstream expected;
